@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qmlengine.h"
 
@@ -31,23 +9,24 @@
 #include "qmlengineutils.h"
 
 #include <debugger/breakhandler.h>
+#include <debugger/console/console.h>
 #include <debugger/debuggeractions.h>
 #include <debugger/debuggercore.h>
 #include <debugger/debuggerinternalconstants.h>
-#include <debugger/debuggerruncontrol.h>
 #include <debugger/debuggertooltipmanager.h>
+#include <debugger/debuggertr.h>
 #include <debugger/sourcefileshandler.h>
 #include <debugger/stackhandler.h>
 #include <debugger/threaddata.h>
 #include <debugger/watchhandler.h>
 #include <debugger/watchwindow.h>
-#include <debugger/console/console.h>
 
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/helpmanager.h>
 #include <coreplugin/icore.h>
 
-#include <projectexplorer/applicationlauncher.h>
+#include <projectexplorer/projectnodes.h>
+#include <projectexplorer/projecttree.h>
 
 #include <qmljseditor/qmljseditorconstants.h>
 #include <qmljs/qmljsmodelmanagerinterface.h>
@@ -57,16 +36,17 @@
 #include <texteditor/textdocument.h>
 #include <texteditor/texteditor.h>
 
-#include <app/app_version.h>
-#include <utils/treemodel.h>
 #include <utils/basetreeview.h>
 #include <utils/fileinprojectfinder.h>
+#include <utils/qtcprocess.h>
 #include <utils/qtcassert.h>
+#include <utils/treemodel.h>
 
 #include <QDebug>
 #include <QDir>
 #include <QDockWidget>
 #include <QFileInfo>
+#include <QGuiApplication>
 #include <QHostAddress>
 #include <QJsonArray>
 #include <QJsonDocument>
@@ -95,8 +75,7 @@ using namespace QmlJS;
 using namespace TextEditor;
 using namespace Utils;
 
-namespace Debugger {
-namespace Internal {
+namespace Debugger::Internal {
 
 enum Exceptions
 {
@@ -162,7 +141,7 @@ public:
     void evaluate(const QString expr, qint64 context, const QmlCallback &cb);
     void lookup(const LookupItems &items);
     void backtrace();
-    void updateLocals();
+    void updateLocals(bool focusOnFrame = true);
     void scope(int number, int frameNumber = -1);
     void scripts(int types = 4, const QList<int> ids = QList<int>(),
                  bool includeSource = false, const QVariant filter = QVariant());
@@ -220,7 +199,7 @@ public:
 
     QHash<QString, QTextDocument*> sourceDocuments;
     InteractiveInterpreter interpreter;
-    ApplicationLauncher applicationLauncher;
+    Process process;
     QmlInspectorAgent inspectorAgent;
 
     QList<quint32> queryIds;
@@ -237,6 +216,8 @@ public:
 
     FileInProjectFinder fileFinder;
 
+    bool skipFocusOnNextHandleFrame = false;
+
 private:
     ConsoleItem *constructLogItemTree(const QmlV8ObjectData &objectData, QList<int> &seenHandles);
     void constructChildLogItems(ConsoleItem *item, const QmlV8ObjectData &objectData,
@@ -269,12 +250,17 @@ QmlEngine::QmlEngine()
     connect(stackHandler(), &StackHandler::currentIndexChanged,
             this, &QmlEngine::updateCurrentContext);
 
-    connect(&d->applicationLauncher, &ApplicationLauncher::processExited,
-            this, &QmlEngine::disconnected);
-    connect(&d->applicationLauncher, &ApplicationLauncher::appendMessage,
-            this, &QmlEngine::appMessage);
-    connect(&d->applicationLauncher, &ApplicationLauncher::processStarted,
-            this, &QmlEngine::handleLauncherStarted);
+    connect(&d->process, &Process::readyReadStandardOutput, this, [this] {
+        // FIXME: Redirect to RunControl
+        showMessage(d->process.readAllStandardOutput(), AppOutput);
+    });
+    connect(&d->process, &Process::readyReadStandardError, this, [this] {
+        // FIXME: Redirect to RunControl
+        showMessage(d->process.readAllStandardError(), AppOutput);
+    });
+
+    connect(&d->process, &Process::done, this, &QmlEngine::disconnected);
+    connect(&d->process, &Process::started, this, &QmlEngine::handleLauncherStarted);
 
     debuggerConsole()->populateFileFinder();
     debuggerConsole()->setScriptEvaluator([this](const QString &expr) {
@@ -329,11 +315,6 @@ void QmlEngine::handleLauncherStarted()
     tryToConnect();
 }
 
-void QmlEngine::appMessage(const QString &msg, Utils::OutputFormat /* format */)
-{
-    showMessage(msg, AppOutput); // FIXME: Redirect to RunControl
-}
-
 void QmlEngine::connectionEstablished()
 {
     connect(inspectorView(), &WatchTreeView::currentIndexChanged,
@@ -350,7 +331,7 @@ void QmlEngine::tryToConnect()
     if (state() == EngineRunRequested) {
         if (isDying()) {
             // Probably cpp is being debugged and hence we did not get the output yet.
-            appStartupFailed(tr("No application output received in time"));
+            appStartupFailed(Tr::tr("No application output received in time"));
         } else {
             beginConnection();
         }
@@ -366,8 +347,11 @@ void QmlEngine::beginConnection()
 
     QTC_ASSERT(state() == EngineRunRequested, return);
 
+    QmlDebugConnection *connection = d->connection();
+    if (!connection || connection->isConnected())
+        return;
 
-    QString host = runParameters().qmlServer.host();
+    QString host = runParameters().qmlServer().host();
     // Use localhost as default
     if (host.isEmpty())
         host = QHostAddress(QHostAddress::LocalHost).toString();
@@ -384,12 +368,7 @@ void QmlEngine::beginConnection()
      * the connection will be closed again (instead of returning the "connection refused"
      * error that we expect).
      */
-    int port = runParameters().qmlServer.port();
-
-    QmlDebugConnection *connection = d->connection();
-    if (!connection || connection->isConnected())
-        return;
-
+    const int port = runParameters().qmlServer().port();
     connection->connectToHost(host, port);
 
     //A timeout to check the connection state
@@ -398,6 +377,9 @@ void QmlEngine::beginConnection()
 
 void QmlEngine::connectionStartupFailed()
 {
+    if (isDying())
+        return;
+
     if (d->retryOnConnectFail) {
         // retry after 3 seconds ...
         QTimer::singleShot(3000, this, [this] { beginConnection(); });
@@ -406,8 +388,8 @@ void QmlEngine::connectionStartupFailed()
 
     auto infoBox = new QMessageBox(ICore::dialogParent());
     infoBox->setIcon(QMessageBox::Critical);
-    infoBox->setWindowTitle(Core::Constants::IDE_DISPLAY_NAME);
-    infoBox->setText(tr("Could not connect to the in-process QML debugger."
+    infoBox->setWindowTitle(QGuiApplication::applicationDisplayName());
+    infoBox->setText(Tr::tr("Could not connect to the in-process QML debugger."
                         "\nDo you want to retry?"));
     infoBox->setStandardButtons(QMessageBox::Retry | QMessageBox::Cancel |
                                 QMessageBox::Help);
@@ -422,20 +404,20 @@ void QmlEngine::connectionStartupFailed()
 
 void QmlEngine::appStartupFailed(const QString &errorMessage)
 {
-    QString error = tr("Could not connect to the in-process QML debugger. %1").arg(errorMessage);
+    QString error = Tr::tr("Could not connect to the in-process QML debugger. %1").arg(errorMessage);
 
-    if (companionEngine()) {
+    if (companionEngines().isEmpty()) {
+        debuggerConsole()->printItem(ConsoleItem::WarningType, error);
+    } else {
         auto infoBox = new QMessageBox(ICore::dialogParent());
         infoBox->setIcon(QMessageBox::Critical);
-        infoBox->setWindowTitle(Core::Constants::IDE_DISPLAY_NAME);
+        infoBox->setWindowTitle(QGuiApplication::applicationDisplayName());
         infoBox->setText(error);
         infoBox->setStandardButtons(QMessageBox::Ok | QMessageBox::Help);
         infoBox->setDefaultButton(QMessageBox::Ok);
         connect(infoBox, &QDialog::finished,
                 this, &QmlEngine::errorMessageBoxFinished);
         infoBox->show();
-    } else {
-        debuggerConsole()->printItem(ConsoleItem::WarningType, error);
     }
 
     notifyEngineRunFailed();
@@ -465,14 +447,15 @@ void QmlEngine::errorMessageBoxFinished(int result)
 
 void QmlEngine::gotoLocation(const Location &location)
 {
-    if (QUrl(location.fileName().toString()).isLocalFile()) { // create QUrl to ensure validity
-        const QString fileName = location.fileName().toString();
+    if (QUrl(location.fileName().toUrlishString()).isLocalFile()) { // create QUrl to ensure validity
+        const QString fileName = location.fileName().toUrlishString();
         // internal file from source files -> show generated .js
         QTC_ASSERT(d->sourceDocuments.contains(fileName), return);
 
-        QString titlePattern = tr("JS Source for %1").arg(fileName);
+        QString titlePattern = Tr::tr("JS Source for %1").arg(fileName);
         //Check if there are open documents with the same title
-        foreach (IDocument *document, DocumentModel::openedDocuments()) {
+        const QList<IDocument *> documents = DocumentModel::openedDocuments();
+        for (IDocument *document: documents) {
             if (document->displayName() == titlePattern) {
                 EditorManager::activateEditorForDocument(document);
                 return;
@@ -493,54 +476,29 @@ void QmlEngine::gotoLocation(const Location &location)
 
 void QmlEngine::closeConnection()
 {
-    if (d->connectionTimer.isActive()) {
-        d->connectionTimer.stop();
-    } else {
-        if (QmlDebugConnection *connection = d->connection())
-            connection->close();
-    }
+    d->automaticConnect = false;
+    d->retryOnConnectFail = false;
+    d->connectionTimer.stop();
+    if (QmlDebugConnection *connection = d->connection())
+        connection->close();
 }
 
-void QmlEngine::runEngine()
+void QmlEngine::startProcess()
 {
-    // we won't get any debug output
-    if (!terminal()) {
-        d->retryOnConnectFail = true;
-        d->automaticConnect = true;
-    }
+    if (d->process.isRunning())
+        return;
 
-    QTC_ASSERT(state() == EngineRunRequested, qDebug() << state());
-
-    if (isPrimaryEngine()) {
-        // QML only.
-        if (runParameters().startMode == AttachToRemoteServer)
-            tryToConnect();
-        else if (runParameters().startMode == AttachToRemoteProcess)
-            beginConnection();
-        else
-            startApplicationLauncher();
-    } else {
-        tryToConnect();
-    }
+    d->process.setCommand(runParameters().inferior().command);
+    d->process.setWorkingDirectory(runParameters().inferior().workingDirectory);
+    d->process.setEnvironment(runParameters().inferior().environment);
+    showMessage(Tr::tr("Starting %1").arg(d->process.commandLine().toUserOutput()),
+        NormalMessageFormat);
+    d->process.start();
 }
 
-void QmlEngine::startApplicationLauncher()
+void QmlEngine::stopProcess()
 {
-    if (!d->applicationLauncher.isRunning()) {
-        const Runnable runnable = runParameters().inferior;
-        showMessage(tr("Starting %1").arg(runnable.commandLine().toUserOutput()),
-                    NormalMessageFormat);
-        d->applicationLauncher.start(runnable);
-    }
-}
-
-void QmlEngine::stopApplicationLauncher()
-{
-    if (d->applicationLauncher.isRunning()) {
-        disconnect(&d->applicationLauncher, &ApplicationLauncher::processExited,
-                   this, &QmlEngine::disconnected);
-        d->applicationLauncher.stop();
-    }
+    d->process.close();
 }
 
 void QmlEngine::shutdownInferior()
@@ -554,8 +512,8 @@ void QmlEngine::shutdownInferior()
     d->runCommand({DISCONNECT});
 
     resetLocation();
-    stopApplicationLauncher();
     closeConnection();
+    stopProcess();
 
     notifyInferiorShutdownFinished();
 }
@@ -567,7 +525,7 @@ void QmlEngine::shutdownEngine()
     debuggerConsole()->setScriptEvaluator(ScriptEvaluator());
 
    // double check (ill engine?):
-    stopApplicationLauncher();
+    stopProcess();
 
     notifyEngineShutdownFinished();
 }
@@ -576,6 +534,27 @@ void QmlEngine::setupEngine()
 {
     notifyEngineSetupOk();
 
+    // we won't get any debug output
+    if (!usesTerminal()) {
+        d->retryOnConnectFail = true;
+        d->automaticConnect = true;
+    }
+
+    QTC_ASSERT(state() == EngineRunRequested, qDebug() << state());
+
+    if (isPrimaryEngine()) {
+        // QML only.
+        const DebuggerStartMode startMode = runParameters().startMode();
+        if (startMode == AttachToQmlServer || startMode == AttachToRemoteServer)
+            tryToConnect();
+        else if (startMode == AttachToRemoteProcess)
+            beginConnection();
+        else
+            startProcess();
+    } else {
+        tryToConnect();
+    }
+
     if (d->automaticConnect)
         beginConnection();
 }
@@ -592,9 +571,13 @@ void QmlEngine::continueInferior()
 
 void QmlEngine::interruptInferior()
 {
+    if (isDying()) {
+        notifyInferiorStopOk();
+        return;
+    }
     showMessage(INTERRUPT, LogInput);
     d->runDirectCommand(INTERRUPT);
-    showStatusMessage(tr("Waiting for JavaScript engine to interrupt on next statement."));
+    showStatusMessage(Tr::tr("Waiting for JavaScript engine to interrupt on next statement."));
 }
 
 void QmlEngine::executeStepIn(bool)
@@ -624,11 +607,11 @@ void QmlEngine::executeStepOver(bool)
 void QmlEngine::executeRunToLine(const ContextData &data)
 {
     QTC_ASSERT(state() == InferiorStopOk, qDebug() << state());
-    showStatusMessage(tr("Run to line %1 (%2) requested...")
-                          .arg(data.lineNumber)
-                          .arg(data.fileName.toString()),
+    showStatusMessage(Tr::tr("Run to line %1 (%2) requested...")
+                          .arg(data.textPosition.line)
+                          .arg(data.fileName.toUserOutput()),
                       5000);
-    d->setBreakpoint(SCRIPTREGEXP, data.fileName.toString(), true, data.lineNumber);
+    d->setBreakpoint(SCRIPTREGEXP, data.fileName.toUrlishString(), true, data.textPosition.line);
     clearExceptionSelection();
     d->continueDebugging(Continue);
 
@@ -679,8 +662,8 @@ void QmlEngine::insertBreakpoint(const Breakpoint &bp)
         d->setExceptionBreak(AllExceptions, requested.enabled);
 
     } else if (requested.type == BreakpointByFileAndLine) {
-        d->setBreakpoint(SCRIPTREGEXP, requested.fileName.toString(),
-                         requested.enabled, requested.lineNumber, 0,
+        d->setBreakpoint(SCRIPTREGEXP, requested.fileName.toUrlishString(),
+                         requested.enabled, requested.textPosition.line, 0,
                          requested.condition, requested.ignoreCount);
 
     } else if (requested.type == BreakpointOnQmlSignalEmit) {
@@ -737,8 +720,8 @@ void QmlEngine::updateBreakpoint(const Breakpoint &bp)
         d->changeBreakpoint(bp, requested.enabled);
     } else {
         d->clearBreakpoint(bp);
-        d->setBreakpoint(SCRIPTREGEXP, requested.fileName.toString(),
-                         requested.enabled, requested.lineNumber, 0,
+        d->setBreakpoint(SCRIPTREGEXP, requested.fileName.toUrlishString(),
+                         requested.enabled, requested.textPosition.line, 0,
                          requested.condition, requested.ignoreCount);
         d->breakpointsSync.insert(d->sequence, bp);
     }
@@ -758,7 +741,7 @@ bool QmlEngine::acceptsBreakpoint(const BreakpointParameters &bp) const
     return bp.isQmlFileAndLineBreakpoint();
 }
 
-void QmlEngine::loadSymbols(const QString &moduleName)
+void QmlEngine::loadSymbols(const FilePath &moduleName)
 {
     Q_UNUSED(moduleName)
 }
@@ -781,7 +764,7 @@ void QmlEngine::updateAll()
     d->updateLocals();
 }
 
-void QmlEngine::requestModuleSymbols(const QString &moduleName)
+void QmlEngine::requestModuleSymbols(const FilePath &moduleName)
 {
     Q_UNUSED(moduleName)
 }
@@ -798,9 +781,15 @@ void QmlEngine::assignValueInDebugger(WatchItem *item,
     const QString &expression, const QVariant &editValue)
 {
     if (!expression.isEmpty()) {
-        QVariant value = (editValue.type() == QVariant::String)
-                ? QVariant('"' + editValue.toString().replace('"', "\\\"") + '"')
-                : editValue;
+        QTC_CHECK(editValue.typeId() == QMetaType::QString);
+        QVariant value;
+        QString val = editValue.toString();
+        if (item->type == "boolean")
+            value = val != "false" && val != "0";
+        else if (item->type == "number")
+            value = val.toDouble();
+        else
+            value = QString('"' + val.replace('"', "\\\"") + '"');
 
         if (item->isInspect()) {
             d->inspectorAgent.assignValue(item, expression, value);
@@ -808,9 +797,9 @@ void QmlEngine::assignValueInDebugger(WatchItem *item,
             StackHandler *handler = stackHandler();
             QString exp = QString("%1 = %2;").arg(expression).arg(value.toString());
             if (handler->isContentsValid() && handler->currentFrame().isUsable()) {
-                d->evaluate(exp, -1, [this](const QVariantMap &) { d->updateLocals(); });
+                d->evaluate(exp, -1, [this](const QVariantMap &) { d->updateLocals(false); });
             } else {
-                showMessage(tr("Cannot evaluate %1 in current stack frame.")
+                showMessage(Tr::tr("Cannot evaluate %1 in current stack frame.")
                             .arg(expression), ConsoleOutput);
             }
         }
@@ -862,16 +851,15 @@ bool compareConsoleItems(const ConsoleItem *a, const ConsoleItem *b)
     return a->text() < b->text();
 }
 
-static ConsoleItem *constructLogItemTree(const QVariant &result,
-                                         const QString &key = QString())
+static ConsoleItem *constructLogItemTree(const QVariant &result, const QString &key = {})
 {
-    bool sorted = boolSetting(SortStructMembers);
+    const bool sorted = settings().sortStructMembers();
     if (!result.isValid())
         return nullptr;
 
     QString text;
     ConsoleItem *item = nullptr;
-    if (result.type() == QVariant::Map) {
+    if (result.typeId() == QMetaType::QVariantMap) {
         if (key.isEmpty())
             text = "Object";
         else
@@ -890,12 +878,12 @@ static ConsoleItem *constructLogItemTree(const QVariant &result,
             std::sort(children.begin(), children.end(), compareConsoleItems);
 
         item = new ConsoleItem(ConsoleItem::DefaultType, text);
-        foreach (ConsoleItem *child, children) {
+        for (ConsoleItem *child : std::as_const(children)) {
             if (child)
                 item->appendChild(child);
         }
 
-    } else if (result.type() == QVariant::List) {
+    } else if (result.typeId() == QMetaType::QVariantList) {
         if (key.isEmpty())
             text = "List";
         else
@@ -910,11 +898,11 @@ static ConsoleItem *constructLogItemTree(const QVariant &result,
             std::sort(children.begin(), children.end(), compareConsoleItems);
 
         item = new ConsoleItem(ConsoleItem::DefaultType, text);
-        foreach (ConsoleItem *child, children) {
+        for (ConsoleItem *child : std::as_const(children)) {
             if (child)
                 item->appendChild(child);
         }
-    } else if (result.canConvert(QVariant::String)) {
+    } else if (result.canConvert(QMetaType(QMetaType::QString))) {
         item = new ConsoleItem(ConsoleItem::DefaultType, result.toString());
     } else {
         item = new ConsoleItem(ConsoleItem::DefaultType, "Unknown Value");
@@ -949,18 +937,9 @@ bool QmlEngine::hasCapability(unsigned cap) const
         | AddWatcherCapability;*/
 }
 
-void QmlEngine::quitDebugger()
-{
-    d->automaticConnect = false;
-    d->retryOnConnectFail = false;
-    stopApplicationLauncher();
-    closeConnection();
-}
-
 void QmlEngine::doUpdateLocals(const UpdateParameters &params)
 {
-    Q_UNUSED(params)
-    d->updateLocals();
+    d->updateLocals(params.qmlFocusOnFrame);
 }
 
 Context QmlEngine::languageContext() const
@@ -970,7 +949,9 @@ Context QmlEngine::languageContext() const
 
 void QmlEngine::disconnected()
 {
-    showMessage(tr("QML Debugger disconnected."), StatusBar);
+    if (isDying())
+        return;
+    showMessage(Tr::tr("QML Debugger disconnected."), StatusBar);
     notifyInferiorExited();
 }
 
@@ -1007,8 +988,8 @@ void QmlEngine::updateCurrentContext()
         return;
     }
 
-    debuggerConsole()->setContext(tr("Context:") + ' '
-                                  + (context.isEmpty() ? tr("Global QML Context") : context));
+    debuggerConsole()->setContext(Tr::tr("Context:") + ' '
+                                  + (context.isEmpty() ? Tr::tr("Global QML Context") : context));
 }
 
 void QmlEngine::executeDebuggerCommand(const QString &command)
@@ -1046,7 +1027,7 @@ bool QmlEngine::companionPreventsActions() const
 {
     // We need a C++ Engine in a Running state to do anything sensible
     // as otherwise the debugger services in the debuggee are unresponsive.
-    if (DebuggerEngine *companion = companionEngine())
+    if (DebuggerEngine *companion = companionEngines().value(0))
         return companion->state() != InferiorRunOk;
 
     return false;
@@ -1095,9 +1076,10 @@ void QmlEnginePrivate::updateScriptSource(const QString &fileName, int lineOffse
     }
 
     //update open editors
-    QString titlePattern = QCoreApplication::translate("QmlEngine", "JS Source for %1").arg(fileName);
+    QString titlePattern = Tr::tr("JS Source for %1").arg(fileName);
     //Check if there are open editors with the same title
-    foreach (IDocument *doc, DocumentModel::openedDocuments()) {
+    const QList<IDocument *> documents = DocumentModel::openedDocuments();
+    for (IDocument *doc: documents) {
         if (doc->displayName() == titlePattern) {
             updateDocument(doc, document);
             break;
@@ -1116,7 +1098,7 @@ void QmlEngine::connectionFailed()
 {
     // this is only an error if we are already connected and something goes wrong.
     if (isConnected()) {
-        showMessage(tr("QML Debugger: Connection failed."), StatusBar);
+        showMessage(Tr::tr("QML Debugger: Connection failed."), StatusBar);
         notifyInferiorSpontaneousStop();
         notifyInferiorIll();
     } else {
@@ -1143,6 +1125,8 @@ bool QmlEngine::isConnected() const
 
 void QmlEngine::showConnectionStateMessage(const QString &message)
 {
+    if (isDying())
+        return;
     showMessage("QML Debugger: " + message, LogStatus);
 }
 
@@ -1309,13 +1293,14 @@ void QmlEnginePrivate::backtrace()
     runCommand(cmd, CB(handleBacktrace));
 }
 
-void QmlEnginePrivate::updateLocals()
+void QmlEnginePrivate::updateLocals(bool focusOnFrame)
 {
     //    { "seq"       : <number>,
     //      "type"      : "request",
     //      "command"   : "frame",
     //      "arguments" : { "number" : <frame number> }
     //    }
+    skipFocusOnNextHandleFrame = focusOnFrame;
 
     DebuggerCommand cmd(FRAME);
     cmd.arg(NUMBER, stackIndexLookup.value(engine->stackHandler()->currentIndex()));
@@ -1369,9 +1354,9 @@ void QmlEnginePrivate::scripts(int types, const QList<int> ids, bool includeSour
     if (includeSource)
         cmd.arg(INCLUDESOURCE, includeSource);
 
-    if (filter.type() == QVariant::String)
+    if (filter.typeId() == QMetaType::QString)
         cmd.arg(FILTER, filter.toString());
-    else if (filter.type() == QVariant::Int)
+    else if (filter.typeId() == QMetaType::Int)
         cmd.arg(FILTER, filter.toInt());
     else
         QTC_CHECK(!filter.isValid());
@@ -1379,6 +1364,17 @@ void QmlEnginePrivate::scripts(int types, const QList<int> ids, bool includeSour
     runCommand(cmd);
 }
 
+static QString targetFile(const FilePath &original)
+{
+    auto projectTree = ProjectExplorer::ProjectTree::instance();
+    auto node = projectTree->nodeForFile(original);
+
+    if (auto resourceNode = dynamic_cast<ProjectExplorer::ResourceFileNode *>(node))
+        return QLatin1String("qrc:") + resourceNode->qrcPath();
+
+    return original.fileName();
+}
+
 void QmlEnginePrivate::setBreakpoint(const QString type, const QString target,
                                      bool enabled, int line, int column,
                                      const QString condition, int ignoreCount)
@@ -1408,7 +1404,7 @@ void QmlEnginePrivate::setBreakpoint(const QString type, const QString target,
         cmd.arg(ENABLED, enabled);
 
         if (type == SCRIPTREGEXP)
-            cmd.arg(TARGET, Utils::FilePath::fromString(target).fileName());
+            cmd.arg(TARGET, targetFile(FilePath::fromString(target)));
         else
             cmd.arg(TARGET, target);
 
@@ -1641,7 +1637,8 @@ void QmlEnginePrivate::runDirectCommand(const QString &type, const QByteArray &m
 void QmlEnginePrivate::memorizeRefs(const QVariant &refs)
 {
     if (refs.isValid()) {
-        foreach (const QVariant &ref, refs.toList()) {
+        const QList<QVariant> refList = refs.toList();
+        for (const QVariant &ref : refList) {
             const QVariantMap refData = ref.toMap();
             int handle = refData.value(HANDLE).toInt();
             refVals[handle] = extractData(refData);
@@ -1730,7 +1727,7 @@ void QmlEnginePrivate::messageReceived(const QByteArray &data)
                             //The breakpoint requested line should be same as
                             //actual line
                             if (bp && bp->state() != BreakpointInserted) {
-                                bp->setLineNumber(line);
+                                bp->setTextPosition({line, -1});
                                 bp->setPending(false);
                                 engine->notifyBreakpointInsertOk(bp);
                             }
@@ -1805,10 +1802,10 @@ void QmlEnginePrivate::messageReceived(const QByteArray &data)
                             updateScriptSource(name, lineOffset, columnOffset, source);
                         }
 
-                        QMap<QString,QString> files;
-                        foreach (const QString &file, sourceFiles) {
+                        QMap<QString, FilePath> files;
+                        for (const QString &file : std::as_const(sourceFiles)) {
                             QString shortName = file;
-                            QString fullName = engine->toFileInProject(file);
+                            FilePath fullName = engine->toFileInProject(file);
                             files.insert(shortName, fullName);
                         }
 
@@ -1835,6 +1832,15 @@ void QmlEnginePrivate::messageReceived(const QByteArray &data)
                     QList<Breakpoint> v8Breakpoints;
 
                     const QVariantList v8BreakpointIdList = breakData.value("breakpoints").toList();
+                    if (engine->state() != InferiorStopRequested) {
+                        // skip debug break if no breakpoint and we have not done a single step as
+                        // last action - likely stopped in another file with same naming
+                        if (v8BreakpointIdList.isEmpty() && previousStepAction == Continue) {
+                            inferiorStop = false;
+                            continueDebugging(Continue);
+                        }
+                    }
+
                     for (const QVariant &breakpointId : v8BreakpointIdList) {
                         const QString responseId = QString::number(breakpointId.toInt());
                         Breakpoint bp = engine->breakHandler()->findBreakpointByResponseId(responseId);
@@ -1851,15 +1857,15 @@ void QmlEnginePrivate::messageReceived(const QByteArray &data)
 
                         int newColumn = sourceLineText.indexOf('(') + 1;
 
-                        for (const Breakpoint &bp : v8Breakpoints) {
+                        for (const Breakpoint &bp : std::as_const(v8Breakpoints)) {
                             QTC_ASSERT(bp, continue);
                             const BreakpointParameters &params = bp->requestedParameters();
 
                             clearBreakpoint(bp);
                             setBreakpoint(SCRIPTREGEXP,
-                                          params.fileName.toString(),
+                                          params.fileName.toUrlishString(),
                                           params.enabled,
-                                          params.lineNumber,
+                                          params.textPosition.line,
                                           newColumn,
                                           params.condition,
                                           params.ignoreCount);
@@ -1877,20 +1883,20 @@ void QmlEnginePrivate::messageReceived(const QByteArray &data)
 
                     if (inferiorStop) {
                         //Update breakpoint data
-                        for (const Breakpoint &bp : v8Breakpoints) {
+                        for (const Breakpoint &bp : std::as_const(v8Breakpoints)) {
                             QTC_ASSERT(bp, continue);
                             if (bp->functionName().isEmpty()) {
                                 bp->setFunctionName(invocationText);
                             }
                             if (bp->state() != BreakpointInserted) {
-                                bp->setLineNumber(breakData.value("sourceLine").toInt() + 1);
+                                bp->setTextPosition({breakData.value("sourceLine").toInt() + 1, -1});
                                 bp->setPending(false);
                                 engine->notifyBreakpointInsertOk(bp);
                             }
                         }
 
                         if (engine->state() == InferiorRunOk) {
-                            for (const Breakpoint &bp : v8Breakpoints) {
+                            for (const Breakpoint &bp : std::as_const(v8Breakpoints)) {
                                 QTC_ASSERT(bp, continue);
                                 if (breakpointsTemp.contains(bp->responseId()))
                                     clearBreakpoint(bp);
@@ -1909,7 +1915,7 @@ void QmlEnginePrivate::messageReceived(const QByteArray &data)
 
                     const QVariantMap script = body.value("script").toMap();
                     QUrl fileUrl(script.value(NAME).toString());
-                    QString filePath = engine->toFileInProject(fileUrl);
+                    FilePath filePath = engine->toFileInProject(fileUrl);
 
                     const QVariantMap exception = body.value("exception").toMap();
                     QString errorMessage = exception.value("text").toString();
@@ -2033,14 +2039,14 @@ StackFrame QmlEnginePrivate::extractStackFrame(const QVariant &bodyVal)
     }
 
     auto extractString = [this](const QVariant &item) {
-        return ((item.type() == QVariant::String) ? item : extractData(item).value).toString();
+        return (item.typeId() == QMetaType::QString ? item : extractData(item).value).toString();
     };
 
     stackFrame.function = extractString(body.value("func"));
     if (stackFrame.function.isEmpty())
-        stackFrame.function = QCoreApplication::translate("QmlEngine", "Anonymous Function");
+        stackFrame.function = Tr::tr("Anonymous Function");
     stackFrame.file = engine->toFileInProject(extractString(body.value("script")));
-    stackFrame.usable = QFileInfo(stackFrame.file).isReadable();
+    stackFrame.usable = stackFrame.file.isReadableFile();
     stackFrame.receiver = extractString(body.value("receiver"));
     stackFrame.line = body.value("line").toInt() + 1;
 
@@ -2132,11 +2138,13 @@ void QmlEnginePrivate::handleFrame(const QVariantMap &response)
         currentFrameScopes.append(scopeIndex);
         this->scope(scopeIndex);
     }
-    engine->gotoLocation(stackHandler->currentFrame());
+
+    if (skipFocusOnNextHandleFrame)
+        engine->gotoLocation(stackHandler->currentFrame());
 
     // Send watchers list
     if (stackHandler->isContentsValid() && stackHandler->currentFrame().isUsable()) {
-        const QStringList watchers = watchHandler->watchedExpressions();
+        const QStringList watchers = WatchHandler::watchedExpressions();
         for (const QString &exp : watchers) {
             const QString iname = watchHandler->watcherName(exp);
             evaluate(exp, -1, [this, iname, exp](const QVariantMap &response) {
@@ -2195,10 +2203,10 @@ void QmlEnginePrivate::handleScope(const QVariantMap &response)
         setWatchItemHasChildren(item.get(), localData.hasChildren());
 
         if (localData.value.isValid() || item->wantsChildren || localData.expectedProperties == 0) {
-            WatchHandler *watchHander = engine->watchHandler();
-            if (watchHander->isExpandedIName(item->iname))
+            WatchHandler *watchHandler = engine->watchHandler();
+            if (watchHandler->isExpandedIName(item->iname))
                 itemsToLookup.insert(int(item->id), {item->iname, item->name, item->exp});
-            watchHander->insertItem(item.release());
+            watchHandler->insertItem(item.release());
         } else {
             itemsToLookup.insert(int(item->id), {item->iname, item->name, item->exp});
         }
@@ -2229,10 +2237,10 @@ void QmlEnginePrivate::constructChildLogItems(ConsoleItem *item, const QmlV8Obje
     for (const QVariant &property : objectData.properties)
         *(it++) = constructLogItemTree(extractData(property), seenHandles);
 
-    if (boolSetting(SortStructMembers))
+    if (settings().sortStructMembers())
         std::sort(children.begin(), children.end(), compareConsoleItems);
 
-    foreach (ConsoleItem *child, children)
+    for (ConsoleItem *child : std::as_const(children))
         item->appendChild(child);
 }
 
@@ -2314,7 +2322,6 @@ void QmlEnginePrivate::insertSubItems(WatchItem *parent, const QVariantList &pro
     QTC_ASSERT(parent, return);
     LookupItems itemsToLookup;
 
-    const QSet<QString> expandedINames = engine->watchHandler()->expandedINames();
     for (const QVariant &property : properties) {
         QmlV8ObjectData propertyData = extractData(property);
         std::unique_ptr<WatchItem> item(new WatchItem);
@@ -2336,13 +2343,13 @@ void QmlEnginePrivate::insertSubItems(WatchItem *parent, const QVariantList &pro
         item->id = propertyData.handle;
         item->type = propertyData.type;
         item->value = propertyData.value.toString();
-        if (item->type.isEmpty() || expandedINames.contains(item->iname))
+        if (item->type.isEmpty() || engine->watchHandler()->isExpandedIName(item->iname))
             itemsToLookup.insert(propertyData.handle, {item->iname, item->name, item->exp});
         setWatchItemHasChildren(item.get(), propertyData.hasChildren());
         parent->appendChild(item.release());
     }
 
-    if (boolSetting(SortStructMembers)) {
+    if (settings().sortStructMembers()) {
         parent->sortChildren([](const WatchItem *item1, const WatchItem *item2) {
             return item1->name < item2->name;
         });
@@ -2358,7 +2365,7 @@ void QmlEnginePrivate::handleExecuteDebuggerCommand(const QVariantMap &response)
         debuggerConsole()->printItem(constructLogItemTree(extractData(response.value(BODY))));
 
         // Update the locals
-        foreach (int index, currentFrameScopes)
+        for (int index : std::as_const(currentFrameScopes))
             scope(index);
     } else {
         debuggerConsole()->printItem(new ConsoleItem(ConsoleItem::ErrorType,
@@ -2382,23 +2389,22 @@ void QmlEnginePrivate::handleLookup(const QVariantMap &response)
     for (const QString &handleString : handlesList) {
         const int handle = handleString.toInt();
         const QmlV8ObjectData bodyObjectData = extractData(body.value(handleString));
-        const QList<LookupData> vals = currentlyLookingUp.values(handle);
+        const LookupData res = currentlyLookingUp.value(handle);
         currentlyLookingUp.remove(handle);
-        for (const LookupData &res : vals) {
-            auto item = new WatchItem;
-            item->exp = res.exp;
-            item->iname = res.iname;
-            item->name = res.name;
-            item->id = handle;
 
-            item->type = bodyObjectData.type;
-            item->value = bodyObjectData.value.toString();
+        auto item = new WatchItem;
+        item->exp = res.exp;
+        item->iname = res.iname;
+        item->name = res.name;
+        item->id = handle;
 
-            setWatchItemHasChildren(item, bodyObjectData.hasChildren());
-            insertSubItems(item, bodyObjectData.properties);
+        item->type = bodyObjectData.type;
+        item->value = bodyObjectData.value.toString();
 
-            engine->watchHandler()->insertItem(item);
-        }
+        setWatchItemHasChildren(item, bodyObjectData.hasChildren());
+        insertSubItems(item, bodyObjectData.properties);
+
+        engine->watchHandler()->insertItem(item);
     }
     checkForFinishedUpdate();
 }
@@ -2411,7 +2417,7 @@ void QmlEnginePrivate::stateChanged(State state)
         BreakpointManager::claimBreakpointsForEngine(engine);
 
         // Since the breakpoint claiming is deferred, we need to also defer the connecting
-        QTimer::singleShot(0, this, [this]() {
+        QTimer::singleShot(0, this, [this] {
             /// Start session.
             flushSendBuffer();
             QJsonObject parameters;
@@ -2434,21 +2440,21 @@ void QmlEnginePrivate::handleVersion(const QVariantMap &response)
 void QmlEnginePrivate::flushSendBuffer()
 {
     QTC_ASSERT(state() == Enabled, return);
-    foreach (const QByteArray &msg, sendBuffer)
+    for (const QByteArray &msg : std::as_const(sendBuffer))
         sendMessage(msg);
     sendBuffer.clear();
 }
 
-QString QmlEngine::toFileInProject(const QUrl &fileUrl)
+FilePath QmlEngine::toFileInProject(const QUrl &fileUrl)
 {
     // make sure file finder is properly initialized
     const DebuggerRunParameters &rp = runParameters();
-    d->fileFinder.setProjectDirectory(rp.projectSourceDirectory);
-    d->fileFinder.setProjectFiles(rp.projectSourceFiles);
-    d->fileFinder.setAdditionalSearchDirectories(rp.additionalSearchDirectories);
-    d->fileFinder.setSysroot(rp.sysRoot);
+    d->fileFinder.setProjectDirectory(rp.projectSourceDirectory());
+    d->fileFinder.setProjectFiles(rp.projectSourceFiles());
+    d->fileFinder.setAdditionalSearchDirectories(rp.additionalSearchDirectories());
+    d->fileFinder.setSysroot(rp.sysRoot());
 
-    return d->fileFinder.findFile(fileUrl).first().toString();
+    return d->fileFinder.findFile(fileUrl).constFirst();
 }
 
 DebuggerEngine *createQmlEngine()
@@ -2456,5 +2462,4 @@ DebuggerEngine *createQmlEngine()
     return new QmlEngine;
 }
 
-} // Internal
-} // Debugger
+} // Debugger::Internal
