@@ -1,80 +1,62 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "genericproject.h"
 
-#include "genericbuildconfiguration.h"
-#include "genericmakestep.h"
 #include "genericprojectconstants.h"
+#include "genericprojectmanagertr.h"
 
+#include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/documentmanager.h>
 #include <coreplugin/icontext.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
 
-#include <cpptools/cppprojectupdaterinterface.h>
-
-#include <extensionsystem/pluginmanager.h>
-
 #include <projectexplorer/abi.h>
 #include <projectexplorer/buildinfo.h>
 #include <projectexplorer/buildsteplist.h>
 #include <projectexplorer/buildsystem.h>
 #include <projectexplorer/customexecutablerunconfiguration.h>
 #include <projectexplorer/deploymentdata.h>
+#include <projectexplorer/environmentkitaspect.h>
 #include <projectexplorer/headerpath.h>
-#include <projectexplorer/kitinformation.h>
+#include <projectexplorer/kitmanager.h>
 #include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/projectexplorertr.h>
+#include <projectexplorer/projectmanager.h>
 #include <projectexplorer/projectnodes.h>
+#include <projectexplorer/projectnodes.h>
+#include <projectexplorer/projecttree.h>
+#include <projectexplorer/projectupdater.h>
 #include <projectexplorer/selectablefilesmodel.h>
 #include <projectexplorer/target.h>
 #include <projectexplorer/taskhub.h>
 
 #include <qtsupport/baseqtversion.h>
 #include <qtsupport/qtcppkitinfo.h>
-#include <qtsupport/qtkitinformation.h>
+#include <qtsupport/qtkitaspect.h>
 
 #include <utils/algorithm.h>
 #include <utils/filesystemwatcher.h>
 #include <utils/fileutils.h>
-#include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
+#include <utils/qtcassert.h>
 
 #include <QDir>
 #include <QFileInfo>
 #include <QHash>
 #include <QMetaObject>
+#include <QPair>
 #include <QSet>
 #include <QStringList>
 
+#include <set>
+
 using namespace Core;
 using namespace ProjectExplorer;
 using namespace Utils;
 
-namespace GenericProjectManager {
-namespace Internal {
+namespace GenericProjectManager::Internal {
 
 enum RefreshOptions {
     Files         = 0x01,
@@ -82,46 +64,14 @@ enum RefreshOptions {
     Everything    = Files | Configuration
 };
 
-////////////////////////////////////////////////////////////////////////////////////
 //
-// GenericProjectFile
+// GenericBuildSystem
 //
-////////////////////////////////////////////////////////////////////////////////////
 
-class GenericProjectFile : public Core::IDocument
+class GenericBuildSystem final : public BuildSystem
 {
 public:
-    GenericProjectFile(GenericProject *parent, const FilePath &fileName, RefreshOptions options)
-        : m_project(parent), m_options(options)
-    {
-        setId("Generic.ProjectFile");
-        setMimeType(Constants::GENERICMIMETYPE);
-        setFilePath(fileName);
-    }
-
-    ReloadBehavior reloadBehavior(ChangeTrigger, ChangeType) const final
-    {
-        return BehaviorSilent;
-    }
-
-    bool reload(QString *errorString, ReloadFlag flag, ChangeType type) override;
-
-private:
-    GenericProject *m_project = nullptr;
-    RefreshOptions m_options;
-};
-
-
-////////////////////////////////////////////////////////////////////////////////////
-//
-// GenericProjectNode
-//
-////////////////////////////////////////////////////////////////////////////////////
-
-class GenericBuildSystem : public BuildSystem
-{
-public:
-    explicit GenericBuildSystem(Target *target);
+    explicit GenericBuildSystem(BuildConfiguration *bc);
     ~GenericBuildSystem();
 
     void triggerParsing() final;
@@ -136,8 +86,12 @@ public:
     }
 
     RemovedFilesFromProject removeFiles(Node *, const FilePaths &filePaths, FilePaths *) final;
-    bool renameFile(Node *, const FilePath &oldFilePath, const FilePath &newFilePath) final;
+    bool renameFiles(
+        Node *,
+        const Utils::FilePairs &filesToRename,
+        Utils::FilePaths *notRenamed) final;
     bool addFiles(Node *, const FilePaths &filePaths, FilePaths *) final;
+    QString name() const final { return QLatin1String("generic"); }
 
     FilePath filesFilePath() const { return ::FilePath::fromString(m_filesFileName); }
 
@@ -147,10 +101,12 @@ public:
     bool saveRawList(const QStringList &rawList, const QString &fileName);
     void parse(RefreshOptions options);
 
-    QStringList processEntries(const QStringList &paths,
+    using SourceFile = QPair<FilePath, QStringList>;
+    using SourceFiles = QList<SourceFile>;
+    SourceFiles processEntries(const QStringList &paths,
                                QHash<QString, QString> *map = nullptr) const;
 
-    Utils::FilePath findCommonSourceRoot();
+    FilePath findCommonSourceRoot();
     void refreshCppCodeModel();
     void updateDeploymentData();
 
@@ -164,56 +120,119 @@ private:
     QString m_cxxflagsFileName;
     QString m_cflagsFileName;
     QStringList m_rawFileList;
-    QStringList m_files;
+    SourceFiles m_files;
     QHash<QString, QString> m_rawListEntries;
     QStringList m_rawProjectIncludePaths;
-    ProjectExplorer::HeaderPaths m_projectIncludePaths;
+    HeaderPaths m_projectIncludePaths;
     QStringList m_cxxflags;
     QStringList m_cflags;
 
-    CppTools::CppProjectUpdaterInterface *m_cppCodeModelUpdater = nullptr;
+    ProjectUpdater *m_cppCodeModelUpdater = nullptr;
 
-    Utils::FileSystemWatcher m_deployFileWatcher;
+    FileSystemWatcher m_deployFileWatcher;
+};
+
+//
+// GenericBuildConfiguration
+//
+
+class GenericBuildConfiguration final : public BuildConfiguration
+{
+public:
+    GenericBuildConfiguration(Target *target, Id id)
+        : BuildConfiguration(target, id), m_buildSystem(new GenericBuildSystem(this))
+    {
+        setConfigWidgetDisplayName(GenericProjectManager::Tr::tr("Generic Manager"));
+        setBuildDirectoryHistoryCompleter("Generic.BuildDir.History");
+
+        setInitializer([this](const BuildInfo &) {
+            buildSteps()->appendStep(Constants::GENERIC_MS_ID);
+            cleanSteps()->appendStep(Constants::GENERIC_MS_ID);
+            updateCacheAndEmitEnvironmentChanged();
+        });
+
+        updateCacheAndEmitEnvironmentChanged();
+    }
+
+    ~GenericBuildConfiguration() { delete m_buildSystem; }
+
+private:
+    void addToEnvironment(Environment &env) const final
+    {
+        QtSupport::QtKitAspect::addHostBinariesToPath(kit(), env);
+    }
+
+    BuildSystem *buildSystem() const { return m_buildSystem; }
+
+    GenericBuildSystem * const m_buildSystem;
+};
+
+class GenericBuildConfigurationFactory final : public BuildConfigurationFactory
+{
+public:
+    GenericBuildConfigurationFactory()
+    {
+        registerBuildConfiguration<GenericBuildConfiguration>
+            ("GenericProjectManager.GenericBuildConfiguration");
+
+        setSupportedProjectType(Constants::GENERICPROJECT_ID);
+        setSupportedProjectMimeTypeName(Constants::GENERICMIMETYPE);
+
+        setBuildGenerator([](const Kit *, const FilePath &projectPath, bool forSetup) {
+            BuildInfo info;
+            info.typeName = ProjectExplorer::Tr::tr("Build");
+            info.buildDirectory = forSetup ? projectPath.absolutePath() : projectPath;
+
+            if (forSetup)  {
+                //: The name of the build configuration created by default for a generic project.
+                info.displayName = ProjectExplorer::Tr::tr("Default");
+            }
+
+            return QList<BuildInfo>{info};
+        });
+    }
 };
 
 
-////////////////////////////////////////////////////////////////////////////////////
 //
 // GenericProject
 //
-////////////////////////////////////////////////////////////////////////////////////
 
 static bool writeFile(const QString &filePath, const QString &contents)
 {
-    Utils::FileSaver saver(Utils::FilePath::fromString(filePath),
-                           QIODevice::Text | QIODevice::WriteOnly);
+    FileSaver saver(FilePath::fromString(filePath),
+                    QIODevice::Text | QIODevice::WriteOnly);
     return saver.write(contents.toUtf8()) && saver.finalize();
 }
 
-GenericProject::GenericProject(const Utils::FilePath &fileName)
-    : Project(Constants::GENERICMIMETYPE, fileName)
+class GenericProject final : public Project
 {
-    setId(Constants::GENERICPROJECT_ID);
-    setProjectLanguages(Context(ProjectExplorer::Constants::CXX_LANGUAGE_ID));
-    setDisplayName(fileName.completeBaseName());
-    setBuildSystemCreator([](Target *t) { return new GenericBuildSystem(t); });
-}
+    Q_OBJECT
 
-GenericBuildSystem::GenericBuildSystem(Target *target)
-    : BuildSystem(target)
-{
-    QObject *projectUpdaterFactory = ExtensionSystem::PluginManager::getObjectByName(
-        "CppProjectUpdaterFactory");
-    if (projectUpdaterFactory) {
-        const bool successFullyCreatedProjectUpdater
-            = QMetaObject::invokeMethod(projectUpdaterFactory,
-                                        "create",
-                                        Q_RETURN_ARG(CppTools::CppProjectUpdaterInterface *,
-                                                     m_cppCodeModelUpdater));
-        QTC_CHECK(successFullyCreatedProjectUpdater);
+public:
+    explicit GenericProject(const FilePath &filePath)
+        : Project(Constants::GENERICMIMETYPE, filePath)
+    {
+        setId(Constants::GENERICPROJECT_ID);
+        setProjectLanguages(Context(ProjectExplorer::Constants::CXX_LANGUAGE_ID));
+        setDisplayName(filePath.completeBaseName());
     }
 
-    connect(target->project(), &Project::projectFileIsDirty, this, [this](const FilePath &p) {
+    void editFilesTriggered();
+    void removeFilesTriggered(const FilePaths &filesToRemove);
+
+private:
+    RestoreResult fromMap(const Store &map, QString *errorMessage) final;
+    DeploymentKnowledge deploymentKnowledge() const final;
+    void configureAsExampleProject(Kit *kit) final;
+};
+
+GenericBuildSystem::GenericBuildSystem(BuildConfiguration *bc)
+    : BuildSystem(bc)
+{
+    m_cppCodeModelUpdater = ProjectUpdaterFactory::createCppProjectUpdater();
+
+    connect(bc->project(), &Project::projectFileIsDirty, this, [this](const FilePath &p) {
         if (p.endsWith(".files"))
             refresh(Files);
         else if (p.endsWith(".includes") || p.endsWith(".config") || p.endsWith(".cxxflags")
@@ -253,13 +272,11 @@ GenericBuildSystem::GenericBuildSystem(Target *target)
     connect(&m_deployFileWatcher, &FileSystemWatcher::fileChanged,
             this, &GenericBuildSystem::updateDeploymentData);
 
-    connect(target, &Target::activeBuildConfigurationChanged, this, [this, target] {
-        if (target == project()->activeTarget())
-            refresh(Everything);
+    connect(bc->target(), &Target::activeBuildConfigurationChanged, this, [this] {
+        refresh(Everything);
     });
-    connect(project(), &Project::activeTargetChanged, this, [this, target] {
-        if (target == project()->activeTarget())
-            refresh(Everything);
+    connect(project(), &Project::activeTargetChanged, this, [this] {
+        refresh(Everything);
     });
 }
 
@@ -305,7 +322,7 @@ bool GenericBuildSystem::saveRawList(const QStringList &rawList, const QString &
     const FilePath filePath = FilePath::fromString(fileName);
     FileChangeBlocker changeGuard(filePath);
     // Make sure we can open the file for writing
-    Utils::FileSaver saver(filePath, QIODevice::Text);
+    FileSaver saver(filePath, QIODevice::Text);
     if (!saver.hasError()) {
         QTextStream stream(saver.file());
         for (const QString &filePath : rawList)
@@ -327,8 +344,8 @@ static void insertSorted(QStringList *list, const QString &value)
 
 bool GenericBuildSystem::addFiles(Node *, const FilePaths &filePaths_, FilePaths *)
 {
-    const QStringList filePaths = Utils::transform(filePaths_, &FilePath::toString);
-    const QDir baseDir(projectDirectory().toString());
+    const QStringList filePaths = Utils::transform(filePaths_, &FilePath::toUrlishString);
+    const QDir baseDir(projectDirectory().toUrlishString());
     QStringList newList = m_rawFileList;
     if (filePaths.size() > m_rawFileList.size()) {
         newList += transform(filePaths, [&baseDir](const QString &p) {
@@ -346,12 +363,13 @@ bool GenericBuildSystem::addFiles(Node *, const FilePaths &filePaths_, FilePaths
     QSet<QString> toAdd;
 
     for (const QString &filePath : filePaths) {
-        const QString directory = QFileInfo(filePath).absolutePath();
-        if (!includes.contains(directory))
+        const QFileInfo fi(filePath);
+        const QString directory = fi.absolutePath();
+        if (fi.fileName() == "include" && !includes.contains(directory))
             toAdd << directory;
     }
 
-    const QDir dir(projectDirectory().toString());
+    const QDir dir(projectDirectory().toUrlishString());
     const auto candidates = toAdd;
     for (const QString &path : candidates) {
         QString relative = dir.relativeFilePath(path);
@@ -372,7 +390,7 @@ RemovedFilesFromProject GenericBuildSystem::removeFiles(Node *, const FilePaths
     QStringList newList = m_rawFileList;
 
     for (const FilePath &filePath : filePaths) {
-        QHash<QString, QString>::iterator i = m_rawListEntries.find(filePath.toString());
+        QHash<QString, QString>::iterator i = m_rawListEntries.find(filePath.toUrlishString());
         if (i != m_rawListEntries.end())
             newList.removeOne(i.value());
     }
@@ -384,7 +402,7 @@ RemovedFilesFromProject GenericBuildSystem::removeFiles(Node *, const FilePaths
 bool GenericBuildSystem::setFiles(const QStringList &filePaths)
 {
     QStringList newList;
-    QDir baseDir(projectDirectory().toString());
+    QDir baseDir(projectDirectory().toUrlishString());
     for (const QString &filePath : filePaths)
         newList.append(baseDir.relativeFilePath(filePath));
     Utils::sort(newList);
@@ -392,31 +410,52 @@ bool GenericBuildSystem::setFiles(const QStringList &filePaths)
     return saveRawFileList(newList);
 }
 
-bool GenericBuildSystem::renameFile(Node *, const FilePath &oldFilePath, const FilePath &newFilePath)
+bool GenericBuildSystem::renameFiles(Node *, const FilePairs &filesToRename, FilePaths *notRenamed)
 {
     QStringList newList = m_rawFileList;
 
-    QHash<QString, QString>::iterator i = m_rawListEntries.find(oldFilePath.toString());
-    if (i != m_rawListEntries.end()) {
-        int index = newList.indexOf(i.value());
-        if (index != -1) {
-            QDir baseDir(projectDirectory().toString());
-            newList.removeAt(index);
-            insertSorted(&newList, baseDir.relativeFilePath(newFilePath.toString()));
+    bool success = true;
+    for (const auto &[oldFilePath, newFilePath] : filesToRename) {
+        const auto fail = [&, oldFilePath = oldFilePath] {
+            success = false;
+            if (notRenamed)
+                *notRenamed << oldFilePath;
+        };
+
+        const auto i = m_rawListEntries.find(oldFilePath.toUrlishString());
+        if (i == m_rawListEntries.end()) {
+            fail();
+            continue;
         }
+
+        const int index = newList.indexOf(i.value());
+        if (index == -1) {
+            fail();
+            continue;
+        }
+
+        QDir baseDir(projectDirectory().toUrlishString());
+        newList.removeAt(index);
+        insertSorted(&newList, baseDir.relativeFilePath(newFilePath.toUrlishString()));
     }
 
-    return saveRawFileList(newList);
+    if (!saveRawFileList(newList)) {
+        success = false;
+        if (notRenamed)
+            *notRenamed = firstPaths(filesToRename);
+    }
+
+    return success;
 }
 
 static QStringList readFlags(const QString &filePath)
 {
     const QStringList lines = readLines(filePath);
     if (lines.isEmpty())
-        return QStringList();
+        return {};
     QStringList flags;
     for (const auto &line : lines)
-        flags.append(ProcessArgs::splitArgs(line));
+        flags.append(ProcessArgs::splitArgs(line, HostOsInfo::hostOs()));
     return flags;
 }
 
@@ -432,19 +471,20 @@ void GenericBuildSystem::parse(RefreshOptions options)
         m_rawProjectIncludePaths = readLines(m_includesFileName);
         QStringList normalPaths;
         QStringList frameworkPaths;
-        for (const QString &rawPath : qAsConst(m_rawProjectIncludePaths)) {
+        const auto baseDir = FilePath::fromString(m_includesFileName).parentDir();
+        for (const QString &rawPath : std::as_const(m_rawProjectIncludePaths)) {
             if (rawPath.startsWith("-F"))
                 frameworkPaths << rawPath.mid(2);
             else
                 normalPaths << rawPath;
         }
-        const auto stringsToHeaderPaths = [this](const QStringList &paths, HeaderPathType type) {
-            return transform<HeaderPaths>(processEntries(paths),
-                                          [type](const QString &p) { return HeaderPath(p, type);
+        const auto expandedPaths = [this](const QStringList &paths) {
+            return Utils::transform(processEntries(paths), [](const auto &pair) {
+                return pair.first;
             });
         };
-        m_projectIncludePaths = stringsToHeaderPaths(normalPaths, HeaderPathType::User);
-        m_projectIncludePaths << stringsToHeaderPaths(frameworkPaths, HeaderPathType::Framework);
+        m_projectIncludePaths = toUserHeaderPaths(expandedPaths(normalPaths));
+        m_projectIncludePaths << toFrameworkHeaderPaths(expandedPaths(frameworkPaths));
         m_cxxflags = readFlags(m_cxxflagsFileName);
         m_cflags = readFlags(m_cflagsFileName);
     }
@@ -455,8 +495,9 @@ FilePath GenericBuildSystem::findCommonSourceRoot()
     if (m_files.isEmpty())
         return FilePath::fromFileInfo(QFileInfo(m_filesFileName));
 
-    QString root = m_files.front();
-    for (const QString &item : qAsConst(m_files)) {
+    QString root = m_files.front().first.toUrlishString();
+    for (const SourceFile &sourceFile : std::as_const(m_files)) {
+        const QString item = sourceFile.first.toUrlishString();
         if (root.length() > item.length())
             root.truncate(item.length());
 
@@ -472,6 +513,11 @@ FilePath GenericBuildSystem::findCommonSourceRoot()
 
 void GenericBuildSystem::refresh(RefreshOptions options)
 {
+    // TODO: This stanza will have to appear in every BuildSystem and should eventually
+    //       be centralized.
+    if (this != project()->activeBuildSystem())
+        return;
+
     ParseGuard guard = guardParsingRun();
     parse(options);
 
@@ -483,11 +529,11 @@ void GenericBuildSystem::refresh(RefreshOptions options)
         FilePath baseDir = findCommonSourceRoot();
 
         std::vector<std::unique_ptr<FileNode>> fileNodes;
-        for (const QString &f : qAsConst(m_files)) {
+        for (const SourceFile &f : std::as_const(m_files)) {
             FileType fileType = FileType::Source; // ### FIXME
-            if (f.endsWith(".qrc"))
+            if (f.first.endsWith(".qrc"))
                 fileType = FileType::Resource;
-            fileNodes.emplace_back(std::make_unique<FileNode>(FilePath::fromString(f), fileType));
+            fileNodes.emplace_back(std::make_unique<FileNode>(f.first, fileType));
         }
         newRoot->addNestedNodes(std::move(fileNodes), baseDir);
 
@@ -520,24 +566,22 @@ void GenericBuildSystem::refresh(RefreshOptions options)
  * The \a map variable is an optional argument that will map the returned
  * absolute paths back to their original \a entries.
  */
-QStringList GenericBuildSystem::processEntries(const QStringList &paths,
-                                              QHash<QString, QString> *map) const
+GenericBuildSystem::SourceFiles GenericBuildSystem::processEntries(
+        const QStringList &paths, QHash<QString, QString> *map) const
 {
     const BuildConfiguration *const buildConfig = target()->activeBuildConfiguration();
 
-    const Utils::Environment buildEnv = buildConfig ? buildConfig->environment()
-                                                    : Utils::Environment::systemEnvironment();
+    const Environment buildEnv = buildConfig ? buildConfig->environment()
+                                             : Environment::systemEnvironment();
 
-    const Utils::MacroExpander *expander = project()->macroExpander();
-    if (buildConfig)
-        expander = buildConfig->macroExpander();
-    else
-        expander = target()->macroExpander();
+    const MacroExpander *expander = buildConfig ? buildConfig->macroExpander()
+                                                : target()->macroExpander();
 
-    const QDir projectDir(projectDirectory().toString());
+    const QDir projectDir(projectDirectory().toUrlishString());
 
     QFileInfo fileInfo;
-    QStringList absolutePaths;
+    SourceFiles sourceFiles;
+    std::set<QString> seenFiles;
     for (const QString &path : paths) {
         QString trimmedPath = path.trimmed();
         if (trimmedPath.isEmpty())
@@ -546,18 +590,28 @@ QStringList GenericBuildSystem::processEntries(const QStringList &paths,
         trimmedPath = buildEnv.expandVariables(trimmedPath);
         trimmedPath = expander->expand(trimmedPath);
 
-        trimmedPath = Utils::FilePath::fromUserInput(trimmedPath).toString();
+        trimmedPath = FilePath::fromUserInput(trimmedPath).toUrlishString();
+
+        QStringList tagsForFile;
+        const int tagListPos = trimmedPath.indexOf('|');
+        if (tagListPos != -1) {
+            tagsForFile = trimmedPath.mid(tagListPos + 1).simplified()
+                    .split(' ', Qt::SkipEmptyParts);
+            trimmedPath = trimmedPath.left(tagListPos).trimmed();
+        }
+
+        if (!seenFiles.insert(trimmedPath).second)
+            continue;
 
         fileInfo.setFile(projectDir, trimmedPath);
         if (fileInfo.exists()) {
             const QString absPath = fileInfo.absoluteFilePath();
-            absolutePaths.append(absPath);
+            sourceFiles.append({FilePath::fromString(absPath), tagsForFile});
             if (map)
                 map->insert(absPath, trimmedPath);
         }
     }
-    absolutePaths.removeDuplicates();
-    return absolutePaths;
+    return sourceFiles;
 }
 
 void GenericBuildSystem::refreshCppCodeModel()
@@ -571,13 +625,21 @@ void GenericBuildSystem::refreshCppCodeModel()
 
     RawProjectPart rpp;
     rpp.setDisplayName(project()->displayName());
-    rpp.setProjectFileLocation(projectFilePath().toString());
+    rpp.setProjectFileLocation(projectFilePath());
     rpp.setQtVersion(kitInfo.projectPartQtVersion);
     rpp.setHeaderPaths(m_projectIncludePaths);
     rpp.setConfigFileName(m_configFileName);
-    rpp.setFlagsForCxx({nullptr, m_cxxflags, projectDirectory().toString()});
-    rpp.setFlagsForC({nullptr, m_cflags, projectDirectory().toString()});
-    rpp.setFiles(m_files);
+    rpp.setFlagsForCxx({nullptr, m_cxxflags, projectDirectory()});
+    rpp.setFlagsForC({nullptr, m_cflags, projectDirectory()});
+
+    static const auto sourceFilesToStringList = [](const SourceFiles &sourceFiles) {
+        return Utils::transform(sourceFiles, [](const SourceFile &f) {
+            return f.first.toUrlishString();
+        });
+    };
+    rpp.setFiles(sourceFilesToStringList(m_files));
+    rpp.setPreCompiledHeaders(sourceFilesToStringList(
+        Utils::filtered(m_files, [](const SourceFile &f) { return f.second.contains("pch"); })));
 
     m_cppCodeModelUpdater->update({project(), kitInfo, activeParseEnvironment(), {rpp}});
 }
@@ -585,24 +647,23 @@ void GenericBuildSystem::refreshCppCodeModel()
 void GenericBuildSystem::updateDeploymentData()
 {
     static const QString fileName("QtCreatorDeployment.txt");
-    Utils::FilePath deploymentFilePath;
+    FilePath deploymentFilePath;
     BuildConfiguration *bc = target()->activeBuildConfiguration();
     if (bc)
         deploymentFilePath = bc->buildDirectory().pathAppended(fileName);
 
-    bool hasDeploymentData = QFileInfo::exists(deploymentFilePath.toString());
+    bool hasDeploymentData = deploymentFilePath.exists();
     if (!hasDeploymentData) {
         deploymentFilePath = projectDirectory().pathAppended(fileName);
-        hasDeploymentData = QFileInfo::exists(deploymentFilePath.toString());
+        hasDeploymentData = deploymentFilePath.exists();
     }
     if (hasDeploymentData) {
         DeploymentData deploymentData;
-        deploymentData.addFilesFromDeploymentFile(deploymentFilePath.toString(),
-                                                  projectDirectory().toString());
+        deploymentData.addFilesFromDeploymentFile(deploymentFilePath, projectDirectory());
         setDeploymentData(deploymentData);
-        if (m_deployFileWatcher.files() != QStringList(deploymentFilePath.toString())) {
-            m_deployFileWatcher.removeFiles(m_deployFileWatcher.files());
-            m_deployFileWatcher.addFile(deploymentFilePath.toString(),
+        if (m_deployFileWatcher.filePaths() != FilePaths{deploymentFilePath}) {
+            m_deployFileWatcher.clear();
+            m_deployFileWatcher.addFile(deploymentFilePath,
                                         FileSystemWatcher::WatchModifiedDate);
         }
     }
@@ -612,12 +673,12 @@ void GenericBuildSystem::removeFiles(const FilePaths &filesToRemove)
 {
     if (removeFiles(nullptr, filesToRemove, nullptr) == RemovedFilesFromProject::Error) {
         TaskHub::addTask(BuildSystemTask(Task::Error,
-                                         GenericProject::tr("Project files list update failed."),
+                                         Tr::tr("Project files list update failed."),
                                          filesFilePath()));
     }
 }
 
-Project::RestoreResult GenericProject::fromMap(const QVariantMap &map, QString *errorMessage)
+Project::RestoreResult GenericProject::fromMap(const Store &map, QString *errorMessage)
 {
     const RestoreResult result = Project::fromMap(map, errorMessage);
     if (result != RestoreResult::Ok)
@@ -640,18 +701,18 @@ Project::RestoreResult GenericProject::fromMap(const QVariantMap &map, QString *
             t->addRunConfiguration(new CustomExecutableRunConfiguration(t));
     }
 
-    if (Target *t = activeTarget())
-        static_cast<GenericBuildSystem *>(t->buildSystem())->refresh(Everything);
+    if (auto bs = activeBuildSystem())
+        static_cast<GenericBuildSystem *>(bs)->refresh(Everything);
 
     return RestoreResult::Ok;
 }
 
-ProjectExplorer::DeploymentKnowledge GenericProject::deploymentKnowledge() const
+DeploymentKnowledge GenericProject::deploymentKnowledge() const
 {
     return DeploymentKnowledge::Approximative;
 }
 
-void GenericProject::configureAsExampleProject(ProjectExplorer::Kit *kit)
+void GenericProject::configureAsExampleProject(Kit *kit)
 {
     QList<BuildInfo> infoList;
     const QList<Kit *> kits(kit != nullptr ? QList<Kit *>({kit}) : KitManager::kits());
@@ -659,9 +720,9 @@ void GenericProject::configureAsExampleProject(ProjectExplorer::Kit *kit)
         if (auto factory = BuildConfigurationFactory::find(k, projectFilePath())) {
             for (int i = 0; i < 5; ++i) {
                 BuildInfo buildInfo;
-                buildInfo.displayName = tr("Build %1").arg(i + 1);
+                buildInfo.displayName = Tr::tr("Build %1").arg(i + 1);
                 buildInfo.factory = factory;
-                buildInfo.kitId = kit->id();
+                buildInfo.kitId = k->id();
                 buildInfo.buildDirectory = projectFilePath();
                 infoList << buildInfo;
             }
@@ -670,35 +731,57 @@ void GenericProject::configureAsExampleProject(ProjectExplorer::Kit *kit)
     setup(infoList);
 }
 
-bool GenericProjectFile::reload(QString *errorString, IDocument::ReloadFlag flag, IDocument::ChangeType type)
-{
-    Q_UNUSED(errorString)
-    Q_UNUSED(flag)
-    Q_UNUSED(type)
-    if (Target *t = m_project->activeTarget())
-        static_cast<GenericBuildSystem *>(t->buildSystem())->refresh(m_options);
-
-    return true;
-}
-
 void GenericProject::editFilesTriggered()
 {
     SelectableFilesDialogEditFiles sfd(projectDirectory(),
                                        files(Project::AllFiles),
                                        ICore::dialogParent());
     if (sfd.exec() == QDialog::Accepted) {
-        if (Target *t = activeTarget()) {
-            auto bs = static_cast<GenericBuildSystem *>(t->buildSystem());
-            bs->setFiles(transform(sfd.selectedFiles(), &FilePath::toString));
-        }
+        if (auto bs = static_cast<GenericBuildSystem *>(activeBuildSystem()))
+            bs->setFiles(transform(sfd.selectedFiles(), &FilePath::toUrlishString));
     }
 }
 
 void GenericProject::removeFilesTriggered(const FilePaths &filesToRemove)
 {
-    if (Target *t = activeTarget())
-        static_cast<GenericBuildSystem *>(t->buildSystem())->removeFiles(filesToRemove);
+    if (const auto bs = activeBuildSystem())
+        static_cast<GenericBuildSystem *>(bs)->removeFiles(filesToRemove);
 }
 
-} // namespace Internal
-} // namespace GenericProjectManager
+void setupGenericProject(QObject *guard)
+{
+    static GenericBuildConfigurationFactory theGenericBuildConfigurationFactory;
+
+    namespace PEC = ProjectExplorer::Constants;
+
+    ProjectManager::registerProjectType<GenericProject>(Constants::GENERICMIMETYPE);
+
+    ActionBuilder editAction(guard, "GenericProjectManager.EditFiles");
+    editAction.setContext(Constants::GENERICPROJECT_ID);
+    editAction.setText(Tr::tr("Edit Files..."));
+    editAction.setCommandAttribute(Command::CA_Hide);
+    editAction.addToContainer(PEC::M_PROJECTCONTEXT, PEC::G_PROJECT_FILES);
+    editAction.addOnTriggered([] {
+        if (auto genericProject = qobject_cast<GenericProject *>(ProjectTree::currentProject()))
+            genericProject->editFilesTriggered();
+    });
+
+    ActionBuilder removeDirAction(guard, "GenericProject.RemoveDir");
+    removeDirAction.setContext(Constants::GENERICPROJECT_ID);
+    removeDirAction.setText(Tr::tr("Remove Directory"));
+    removeDirAction.addToContainer(PEC::M_FOLDERCONTEXT, PEC::G_FOLDER_OTHER);
+    removeDirAction.addOnTriggered([] {
+        const auto folderNode = ProjectTree::currentNode()->asFolderNode();
+        QTC_ASSERT(folderNode, return);
+        const auto project = qobject_cast<GenericProject *>(folderNode->getProject());
+        QTC_ASSERT(project, return);
+        const FilePaths filesToRemove = transform(
+            folderNode->findNodes([](const Node *node) { return node->asFileNode(); }),
+            [](const Node *node) { return node->filePath();});
+        project->removeFilesTriggered(filesToRemove);
+    });
+}
+
+} // GenericProjectManager::Internal
+
+#include "genericproject.moc"
