@@ -1,38 +1,19 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
+#include "qmljseditortr.h"
 #include "qmljsfindreferences.h"
 
-#include <texteditor/basefilefind.h>
-#include <coreplugin/find/searchresultwindow.h>
-#include <extensionsystem/pluginmanager.h>
-#include <utils/filesearch.h>
-#include <coreplugin/progressmanager/progressmanager.h>
-#include <coreplugin/progressmanager/futureprogress.h>
 #include <coreplugin/editormanager/editormanager.h>
+#include <coreplugin/find/searchresultwindow.h>
 #include <coreplugin/icore.h>
+#include <coreplugin/progressmanager/futureprogress.h>
+#include <coreplugin/progressmanager/progressmanager.h>
+#include <extensionsystem/pluginmanager.h>
+#include <texteditor/basefilefind.h>
+#include <utils/algorithm.h>
+#include <utils/async.h>
+#include <utils/filesearch.h>
 
 #include <qmljs/qmljsmodelmanagerinterface.h>
 #include <qmljs/qmljsbind.h>
@@ -45,18 +26,9 @@
 #include <qmljs/parser/qmljsast_p.h>
 #include <qmljstools/qmljsmodelmanager.h>
 
-#include "qmljseditorconstants.h"
-
-#include <QTime>
-#include <QTimer>
-#include <QtConcurrentRun>
+#include <QDebug>
+#include <QFuture>
 #include <QtConcurrentMap>
-#include <QDir>
-#include <QApplication>
-#include <QLabel>
-#include <utils/runextensions.h>
-
-#include <functional>
 
 using namespace Core;
 using namespace QmlJS;
@@ -70,7 +42,7 @@ namespace {
 class FindUsages: protected Visitor
 {
 public:
-    typedef QList<AST::SourceLocation> Result;
+    using Result = QList<SourceLocation>;
 
     FindUsages(Document::Ptr doc, const ContextPtr &context)
         : _doc(doc)
@@ -90,12 +62,11 @@ public:
     }
 
 protected:
-    void accept(AST::Node *node)
-    { AST::Node::acceptChild(node, this); }
+    void accept(AST::Node *node) { AST::Node::accept(node, this); }
 
     using Visitor::visit;
 
-    virtual bool visit(AST::UiPublicMember *node)
+    bool visit(AST::UiPublicMember *node) override
     {
         if (node->name == _name
                 && _scopeChain.qmlScopeObjects().contains(_scope)) {
@@ -110,7 +81,7 @@ protected:
         return true;
     }
 
-    virtual bool visit(AST::UiObjectDefinition *node)
+    bool visit(AST::UiObjectDefinition *node) override
     {
         _builder.push(node);
         Node::accept(node->initializer, this);
@@ -118,7 +89,13 @@ protected:
         return false;
     }
 
-    virtual bool visit(AST::UiObjectBinding *node)
+    bool visit(AST::TemplateLiteral *el) override
+    {
+        Node::accept(el->expression, this);
+        return true;
+    }
+
+    bool visit(AST::UiObjectBinding *node) override
     {
         if (node->qualifiedId
                 && !node->qualifiedId->next
@@ -133,7 +110,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(AST::UiScriptBinding *node)
+    bool visit(AST::UiScriptBinding *node) override
     {
         if (node->qualifiedId
                 && !node->qualifiedId->next
@@ -151,7 +128,7 @@ protected:
         return true;
     }
 
-    virtual bool visit(AST::UiArrayBinding *node)
+    bool visit(AST::UiArrayBinding *node) override
     {
         if (node->qualifiedId
                 && !node->qualifiedId->next
@@ -162,7 +139,7 @@ protected:
         return true;
     }
 
-    virtual bool visit(AST::IdentifierExpression *node)
+    bool visit(AST::IdentifierExpression *node) override
     {
         if (node->name.isEmpty() || node->name != _name)
             return false;
@@ -193,7 +170,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(AST::FieldMemberExpression *node)
+    bool visit(AST::FieldMemberExpression *node) override
     {
         if (node->name != _name)
             return true;
@@ -209,12 +186,12 @@ protected:
         return true;
     }
 
-    virtual bool visit(AST::FunctionDeclaration *node)
+    bool visit(AST::FunctionDeclaration *node) override
     {
         return visit(static_cast<FunctionExpression *>(node));
     }
 
-    virtual bool visit(AST::FunctionExpression *node)
+    bool visit(AST::FunctionExpression *node) override
     {
         if (node->name == _name) {
             if (checkLookup())
@@ -227,15 +204,20 @@ protected:
         return false;
     }
 
-    virtual bool visit(AST::VariableDeclaration *node)
+    bool visit(AST::PatternElement *node) override
     {
-        if (node->name == _name) {
+        if (node->isVariableDeclaration() && node->bindingIdentifier == _name) {
             if (checkLookup())
                 _usages.append(node->identifierToken);
         }
         return true;
     }
 
+    void throwRecursionDepthError() override
+    {
+        qWarning("Warning: Hit maximum recursion depth while visitin AST in FindUsages");
+    }
+
 private:
     bool contains(const QmlComponentChain *chain)
     {
@@ -249,7 +231,8 @@ private:
         if (root && root->lookupMember(_name, _scopeChain.context()))
             return check(root);
 
-        foreach (const QmlComponentChain *parent, chain->instantiatingComponents()) {
+        const QList<const QmlComponentChain *> parents = chain->instantiatingComponents();
+        for (const QmlComponentChain *parent : parents) {
             if (contains(parent))
                 return true;
         }
@@ -267,7 +250,8 @@ private:
 
     bool checkQmlScope()
     {
-        foreach (const ObjectValue *s, _scopeChain.qmlScopeObjects()) {
+        const QList<const ObjectValue *> scopes = _scopeChain.qmlScopeObjects();
+        for (const ObjectValue *s : scopes) {
             if (check(s))
                 return true;
         }
@@ -276,7 +260,7 @@ private:
 
     bool checkLookup()
     {
-        const ObjectValue *scope = 0;
+        const ObjectValue *scope = nullptr;
         _scopeChain.lookup(_name, &scope);
         return check(scope);
     }
@@ -288,13 +272,13 @@ private:
     ScopeBuilder _builder;
 
     QString _name;
-    const ObjectValue *_scope;
+    const ObjectValue *_scope = nullptr;
 };
 
 class FindTypeUsages: protected Visitor
 {
 public:
-    typedef QList<AST::SourceLocation> Result;
+    using Result = QList<SourceLocation>;
 
     FindTypeUsages(Document::Ptr doc, const ContextPtr &context)
         : _doc(doc)
@@ -315,17 +299,18 @@ public:
     }
 
 protected:
-    void accept(AST::Node *node)
-    { AST::Node::acceptChild(node, this); }
+    void accept(AST::Node *node) { AST::Node::accept(node, this); }
 
     using Visitor::visit;
 
-    virtual bool visit(AST::UiPublicMember *node)
+    bool visit(AST::UiPublicMember *node) override
     {
-        if (node->memberTypeName() == _name){
-            const ObjectValue * tVal = _context->lookupType(_doc.data(), QStringList(_name));
-            if (tVal == _typeValue)
-                _usages.append(node->typeToken);
+        if (UiQualifiedId *memberType = node->memberType) {
+            if (memberType->name == _name) {
+                const ObjectValue * tVal = _context->lookupType(_doc.data(), QStringList(_name));
+                if (tVal == _typeValue)
+                    _usages.append(node->typeToken);
+            }
         }
         if (AST::cast<Block *>(node->statement)) {
             _builder.push(node);
@@ -336,7 +321,7 @@ protected:
         return true;
     }
 
-    virtual bool visit(AST::UiObjectDefinition *node)
+    bool visit(AST::UiObjectDefinition *node) override
     {
         checkTypeName(node->qualifiedTypeNameId);
         _builder.push(node);
@@ -345,7 +330,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(AST::UiObjectBinding *node)
+    bool visit(AST::UiObjectBinding *node) override
     {
         checkTypeName(node->qualifiedTypeNameId);
         _builder.push(node);
@@ -354,7 +339,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(AST::UiScriptBinding *node)
+    bool visit(AST::UiScriptBinding *node) override
     {
         if (AST::cast<Block *>(node->statement)) {
             Node::accept(node->qualifiedId, this);
@@ -366,7 +351,7 @@ protected:
         return true;
     }
 
-    virtual bool visit(AST::IdentifierExpression *node)
+    bool visit(AST::IdentifierExpression *node) override
     {
         if (node->name != _name)
             return false;
@@ -378,7 +363,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(AST::FieldMemberExpression *node)
+    bool visit(AST::FieldMemberExpression *node) override
     {
         if (node->name != _name)
             return true;
@@ -392,12 +377,18 @@ protected:
         return true;
     }
 
-    virtual bool visit(AST::FunctionDeclaration *node)
+    bool visit(AST::TemplateLiteral *el) override
+    {
+        Node::accept(el->expression, this);
+        return true;
+    }
+
+    bool visit(AST::FunctionDeclaration *node) override
     {
         return visit(static_cast<FunctionExpression *>(node));
     }
 
-    virtual bool visit(AST::FunctionExpression *node)
+    bool visit(AST::FunctionExpression *node) override
     {
         Node::accept(node->formals, this);
         _builder.push(node);
@@ -406,13 +397,14 @@ protected:
         return false;
     }
 
-    virtual bool visit(AST::VariableDeclaration *node)
+    bool visit(AST::PatternElement *node) override
     {
-        Node::accept(node->expression, this);
+        if (node->isVariableDeclaration())
+            Node::accept(node->initializer, this);
         return false;
     }
 
-    virtual bool visit(UiImport *ast)
+    bool visit(UiImport *ast) override
     {
         if (ast && ast->importId == _name) {
             const Imports *imp = _context->imports(_doc.data());
@@ -424,6 +416,10 @@ protected:
         return false;
     }
 
+    void throwRecursionDepthError() override
+    {
+        qWarning("Warning: Hit maximum recursion depth while visitin AST in FindTypeUsages");
+    }
 
 private:
     bool checkTypeName(UiQualifiedId *id)
@@ -448,7 +444,7 @@ private:
     ScopeBuilder _builder;
 
     QString _name;
-    const ObjectValue *_typeValue;
+    const ObjectValue *_typeValue = nullptr;
 };
 
 class FindTargetExpression: protected Visitor
@@ -467,8 +463,8 @@ public:
     void operator()(quint32 offset)
     {
         _name.clear();
-        _scope = 0;
-        _objectNode = 0;
+        _scope = nullptr;
+        _objectNode = nullptr;
         _offset = offset;
         _typeKind = ExpKind;
         if (_doc)
@@ -494,12 +490,11 @@ public:
     }
 
 protected:
-    void accept(AST::Node *node)
-    { AST::Node::acceptChild(node, this); }
+    void accept(AST::Node *node) { AST::Node::accept(node, this); }
 
     using Visitor::visit;
 
-    virtual bool preVisit(Node *node)
+    bool preVisit(Node *node) override
     {
         if (Statement *stmt = node->statementCast())
             return containsOffset(stmt->firstSourceLocation(), stmt->lastSourceLocation());
@@ -510,7 +505,7 @@ protected:
         return true;
     }
 
-    virtual bool visit(IdentifierExpression *node)
+    bool visit(IdentifierExpression *node) override
     {
         if (containsOffset(node->identifierToken)) {
             _name = node->name.toString();
@@ -524,7 +519,7 @@ protected:
         return true;
     }
 
-    virtual bool visit(FieldMemberExpression *node)
+    bool visit(FieldMemberExpression *node) override
     {
         if (containsOffset(node->identifierToken)) {
             setScope(node->base);
@@ -547,17 +542,23 @@ protected:
         return true;
     }
 
-    virtual bool visit(UiScriptBinding *node)
+    bool visit(AST::TemplateLiteral *el) override
+    {
+        Node::accept(el->expression, this);
+        return true;
+    }
+
+    bool visit(UiScriptBinding *node) override
     {
         return !checkBindingName(node->qualifiedId);
     }
 
-    virtual bool visit(UiArrayBinding *node)
+    bool visit(UiArrayBinding *node) override
     {
         return !checkBindingName(node->qualifiedId);
     }
 
-    virtual bool visit(UiObjectBinding *node)
+    bool visit(UiObjectBinding *node) override
     {
         if ((!checkTypeName(node->qualifiedTypeNameId)) &&
                 (!checkBindingName(node->qualifiedId))) {
@@ -569,7 +570,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(UiObjectDefinition *node)
+    bool visit(UiObjectDefinition *node) override
     {
         if (!checkTypeName(node->qualifiedTypeNameId)) {
             Node *oldObjectNode = _objectNode;
@@ -580,13 +581,13 @@ protected:
         return false;
     }
 
-    virtual bool visit(UiPublicMember *node)
+    bool visit(UiPublicMember *node) override
     {
         if (containsOffset(node->typeToken)){
-            if (node->isValid()) {
-                _name = node->memberTypeName().toString();
+            if (node->defaultToken().isValid()) {
+                _name = node->memberType->name.toString();
                 _targetValue = _scopeChain->context()->lookupType(_doc.data(), QStringList(_name));
-                _scope = 0;
+                _scope = nullptr;
                 _typeKind = TypeKind;
             }
             return false;
@@ -598,12 +599,23 @@ protected:
         return true;
     }
 
-    virtual bool visit(FunctionDeclaration *node)
+    bool visit(UiEnumDeclaration *node) override
+    {
+        if (containsOffset(node->identifierToken)) {
+            _name = node->name.toString();
+            _scope = _doc->bind()->findQmlObject(_objectNode);
+            _targetValue = _scopeChain->context()->lookupType(_doc.data(), QStringList(_name));
+            return false;
+        }
+        return true;
+    }
+
+    bool visit(FunctionDeclaration *node) override
     {
         return visit(static_cast<FunctionExpression *>(node));
     }
 
-    virtual bool visit(FunctionExpression *node)
+    bool visit(FunctionExpression *node) override
     {
         if (containsOffset(node->identifierToken)) {
             _name = node->name.toString();
@@ -612,15 +624,20 @@ protected:
         return true;
     }
 
-    virtual bool visit(VariableDeclaration *node)
+    bool visit(PatternElement *node) override
     {
-        if (containsOffset(node->identifierToken)) {
-            _name = node->name.toString();
+        if (node->isVariableDeclaration() && containsOffset(node->identifierToken)) {
+            _name = node->bindingIdentifier.toString();
             return false;
         }
         return true;
     }
 
+    void throwRecursionDepthError() override
+    {
+        qWarning("Warning: Hit maximum recursion depth visiting AST in FindUsages");
+    }
+
 private:
     bool containsOffset(SourceLocation start, SourceLocation end)
     {
@@ -647,7 +664,7 @@ private:
         for (UiQualifiedId *att = id; att; att = att->next) {
             if (!att->name.isEmpty() && containsOffset(att->identifierToken)) {
                 _targetValue = _scopeChain->context()->lookupType(_doc.data(), id, att->next);
-                _scope = 0;
+                _scope = nullptr;
                 _name = att->name.toString();
                 _typeKind = TypeKind;
                 return true;
@@ -665,13 +682,13 @@ private:
     }
 
     QString _name;
-    const ObjectValue *_scope;
-    const Value *_targetValue;
-    Node *_objectNode;
+    const ObjectValue *_scope = nullptr;
+    const Value *_targetValue = nullptr;
+    Node *_objectNode = nullptr;
     Document::Ptr _doc;
-    const ScopeChain *_scopeChain;
-    quint32 _offset;
-    Kind _typeKind;
+    const ScopeChain *_scopeChain = nullptr;
+    quint32 _offset = 0;
+    Kind _typeKind = ExpKind;
 };
 
 static QString matchingLine(unsigned position, const QString &source)
@@ -683,66 +700,76 @@ static QString matchingLine(unsigned position, const QString &source)
     return source.mid(start, end - start);
 }
 
-class ProcessFile: public std::unary_function<QString, QList<FindReferences::Usage> >
+class ProcessFile
 {
     ContextPtr context;
-    typedef FindReferences::Usage Usage;
-    QString name;
+    using Usage = FindReferences::Usage;
+    const QString name;
     const ObjectValue *scope;
-    QFutureInterface<Usage> *future;
+    QPromise<Usage> &m_promise;
 
 public:
+    // needed by QtConcurrent
+    using argument_type = const QString &;
+    using result_type = QList<Usage>;
+
     ProcessFile(const ContextPtr &context,
-                QString name,
+                const QString &name,
                 const ObjectValue *scope,
-                QFutureInterface<Usage> *future)
-        : context(context), name(name), scope(scope), future(future)
+                QPromise<Usage> &promise)
+        : context(context), name(name), scope(scope), m_promise(promise)
     { }
 
-    QList<Usage> operator()(const QString &fileName)
+    QList<Usage> operator()(const Utils::FilePath &fileName)
     {
         QList<Usage> usages;
-        if (future->isPaused())
-            future->waitForResume();
-        if (future->isCanceled())
+        m_promise.suspendIfRequested();
+        if (m_promise.isCanceled())
             return usages;
+        ModelManagerInterface *modelManager = ModelManagerInterface::instance();
         Document::Ptr doc = context->snapshot().document(fileName);
         if (!doc)
             return usages;
 
         // find all idenfifier expressions, try to resolve them and check if the result is in scope
         FindUsages findUsages(doc, context);
-        FindUsages::Result results = findUsages(name, scope);
-        foreach (const AST::SourceLocation &loc, results)
-            usages.append(Usage(fileName, matchingLine(loc.offset, doc->source()), loc.startLine, loc.startColumn - 1, loc.length));
-        if (future->isPaused())
-            future->waitForResume();
+        const FindUsages::Result results = findUsages(name, scope);
+        for (const SourceLocation &loc : results)
+            usages.append(Usage(modelManager->fileToSource(fileName),
+                                matchingLine(loc.offset, doc->source()),
+                                loc.startLine,
+                                loc.startColumn - 1,
+                                loc.length));
+        m_promise.suspendIfRequested();
         return usages;
     }
 };
 
-class SearchFileForType: public std::unary_function<QString, QList<FindReferences::Usage> >
+class SearchFileForType
 {
     ContextPtr context;
-    typedef FindReferences::Usage Usage;
-    QString name;
+    using Usage = FindReferences::Usage;
+    const QString name;
     const ObjectValue *scope;
-    QFutureInterface<Usage> *future;
+    QPromise<Usage> &m_promise;
 
 public:
+    // needed by QtConcurrent
+    using argument_type = const QString &;
+    using result_type = QList<Usage>;
+
     SearchFileForType(const ContextPtr &context,
-                      QString name,
+                      const QString &name,
                       const ObjectValue *scope,
-                      QFutureInterface<Usage> *future)
-        : context(context), name(name), scope(scope), future(future)
+                      QPromise<Usage> &promise)
+        : context(context), name(name), scope(scope), m_promise(promise)
     { }
 
-    QList<Usage> operator()(const QString &fileName)
+    QList<Usage> operator()(const Utils::FilePath &fileName)
     {
         QList<Usage> usages;
-        if (future->isPaused())
-            future->waitForResume();
-        if (future->isCanceled())
+        m_promise.suspendIfRequested();
+        if (m_promise.isCanceled())
             return usages;
         Document::Ptr doc = context->snapshot().document(fileName);
         if (!doc)
@@ -750,29 +777,32 @@ public:
 
         // find all idenfifier expressions, try to resolve them and check if the result is in scope
         FindTypeUsages findUsages(doc, context);
-        FindTypeUsages::Result results = findUsages(name, scope);
-        foreach (const AST::SourceLocation &loc, results)
+        const FindTypeUsages::Result results = findUsages(name, scope);
+        for (const SourceLocation &loc : results)
             usages.append(Usage(fileName, matchingLine(loc.offset, doc->source()), loc.startLine, loc.startColumn - 1, loc.length));
-        if (future->isPaused())
-            future->waitForResume();
+        m_promise.suspendIfRequested();
         return usages;
     }
 };
 
-class UpdateUI: public std::binary_function<QList<FindReferences::Usage> &, QList<FindReferences::Usage>, void>
+class UpdateUI
 {
-    typedef FindReferences::Usage Usage;
-    QFutureInterface<Usage> *future;
+    using Usage = FindReferences::Usage;
+    QPromise<Usage> &m_promise;
 
 public:
-    UpdateUI(QFutureInterface<Usage> *future): future(future) {}
+    // needed by QtConcurrent
+    using first_argument_type = QList<Usage> &;
+    using second_argument_type = const QList<Usage> &;
+    using result_type = void;
+
+    UpdateUI(QPromise<Usage> &promise): m_promise(promise) {}
 
     void operator()(QList<Usage> &, const QList<Usage> &usages)
     {
-        foreach (const Usage &u, usages)
-            future->reportResult(u);
-
-        future->setProgressValue(future->progressValue() + 1);
+        for (const Usage &u : usages)
+            m_promise.addResult(u);
+        m_promise.setProgressValue(m_promise.future().progressValue() + 1);
     }
 };
 
@@ -786,24 +816,21 @@ FindReferences::FindReferences(QObject *parent)
     connect(&m_watcher, &QFutureWatcherBase::finished, this, &FindReferences::searchFinished);
 }
 
-FindReferences::~FindReferences()
-{
-}
+FindReferences::~FindReferences() = default;
 
-static void find_helper(QFutureInterface<FindReferences::Usage> &future,
-                        const ModelManagerInterface::WorkingCopy workingCopy,
+static void find_helper(QPromise<FindReferences::Usage> &promise,
+                        const ModelManagerInterface::WorkingCopy &workingCopy,
                         Snapshot snapshot,
-                        const QString fileName,
+                        const Utils::FilePath &fileName,
                         quint32 offset,
                         QString replacement)
 {
     // update snapshot from workingCopy to make sure it's up to date
     // ### remove?
     // ### this is a great candidate for map-reduce
-    QHashIterator< QString, QPair<QString, int> > it(workingCopy.all());
-    while (it.hasNext()) {
-        it.next();
-        const QString fileName = it.key();
+    const ModelManagerInterface::WorkingCopy::Table &all = workingCopy.all();
+    for (auto it = all.cbegin(), end = all.cend(); it != end; ++it) {
+        const Utils::FilePath fileName = it.key();
         Document::Ptr oldDoc = snapshot.document(fileName);
         if (oldDoc && oldDoc->editorRevision() == it.value().second)
             continue;
@@ -847,25 +874,26 @@ static void find_helper(QFutureInterface<FindReferences::Usage> &future,
     if (!replacement.isNull() && replacement.isEmpty())
         replacement = name;
 
-    QStringList files;
-    foreach (const Document::Ptr &doc, snapshot) {
+    Utils::FilePaths files;
+    for (const Document::Ptr &doc : std::as_const(snapshot)) {
         // ### skip files that don't contain the name token
-        files.append(doc->fileName());
+        files.append(modelManager->fileToSource(doc->fileName()));
     }
+    files = Utils::filteredUnique(files);
 
-    future.setProgressRange(0, files.size());
+    promise.setProgressRange(0, files.size());
 
     // report a dummy usage to indicate the search is starting
-    FindReferences::Usage searchStarting(replacement, name, 0, 0, 0);
+    FindReferences::Usage searchStarting(Utils::FilePath::fromString(replacement), name, 0, 0, 0);
 
     if (findTarget.typeKind() == findTarget.TypeKind){
         const ObjectValue *typeValue = value_cast<ObjectValue>(findTarget.targetValue());
         if (!typeValue)
             return;
-        future.reportResult(searchStarting);
+        promise.addResult(searchStarting);
 
-        SearchFileForType process(context, name, typeValue, &future);
-        UpdateUI reduce(&future);
+        SearchFileForType process(context, name, typeValue, promise);
+        UpdateUI reduce(promise);
 
         QtConcurrent::blockingMappedReduced<QList<FindReferences::Usage> > (files, process, reduce);
     } else {
@@ -877,26 +905,28 @@ static void find_helper(QFutureInterface<FindReferences::Usage> &future,
             return;
         if (!scope->className().isEmpty())
             searchStarting.lineText.prepend(scope->className() + QLatin1Char('.'));
-        future.reportResult(searchStarting);
+        promise.addResult(searchStarting);
 
-        ProcessFile process(context, name, scope, &future);
-        UpdateUI reduce(&future);
+        ProcessFile process(context, name, scope, promise);
+        UpdateUI reduce(promise);
 
         QtConcurrent::blockingMappedReduced<QList<FindReferences::Usage> > (files, process, reduce);
     }
-    future.setProgressValue(files.size());
+    promise.setProgressValue(files.size());
 }
 
-void FindReferences::findUsages(const QString &fileName, quint32 offset)
+void FindReferences::findUsages(const Utils::FilePath &fileName, quint32 offset)
 {
     ModelManagerInterface *modelManager = ModelManagerInterface::instance();
 
-    QFuture<Usage> result = Utils::runAsync(&find_helper, modelManager->workingCopy(),
+    QFuture<Usage> result = Utils::asyncRun(&find_helper, ModelManagerInterface::workingCopy(),
                                             modelManager->snapshot(), fileName, offset, QString());
     m_watcher.setFuture(result);
+    m_synchronizer.addFuture(result);
 }
 
-void FindReferences::renameUsages(const QString &fileName, quint32 offset,
+void FindReferences::renameUsages(const Utils::FilePath &fileName,
+                                  quint32 offset,
                                   const QString &replacement)
 {
     ModelManagerInterface *modelManager = ModelManagerInterface::instance();
@@ -906,12 +936,14 @@ void FindReferences::renameUsages(const QString &fileName, quint32 offset,
     if (newName.isNull())
         newName = QLatin1String("");
 
-    QFuture<Usage> result = Utils::runAsync(&find_helper, modelManager->workingCopy(),
+    QFuture<Usage> result = Utils::asyncRun(&find_helper, ModelManagerInterface::workingCopy(),
                                             modelManager->snapshot(), fileName, offset, newName);
     m_watcher.setFuture(result);
+    m_synchronizer.addFuture(result);
 }
 
-QList<FindReferences::Usage> FindReferences::findUsageOfType(const QString &fileName, const QString typeName)
+QList<FindReferences::Usage> FindReferences::findUsageOfType(const Utils::FilePath &fileName,
+                                                             const QString &typeName)
 {
     QList<Usage> usages;
     ModelManagerInterface *modelManager = ModelManagerInterface::instance();
@@ -928,11 +960,22 @@ QList<FindReferences::Usage> FindReferences::findUsageOfType(const QString &file
 
     QmlJS::Snapshot snapshot =  modelManager->snapshot();
 
-    foreach (const QmlJS::Document::Ptr &doc, snapshot) {
-        FindTypeUsages findUsages(doc, context);
-        FindTypeUsages::Result results = findUsages(typeName, targetValue);
-        foreach (const AST::SourceLocation &loc, results) {
-            usages.append(Usage(doc->fileName(), matchingLine(loc.offset, doc->source()), loc.startLine, loc.startColumn - 1, loc.length));
+    QSet<Utils::FilePath> docDone;
+    for (const QmlJS::Document::Ptr &doc : std::as_const(snapshot)) {
+        Utils::FilePath sourceFile = modelManager->fileToSource(doc->fileName());
+        if (!Utils::insert(docDone, sourceFile))
+            continue;
+        QmlJS::Document::Ptr sourceDoc = doc;
+        if (sourceFile != doc->fileName())
+            sourceDoc = snapshot.document(sourceFile);
+        FindTypeUsages findUsages(sourceDoc, context);
+        const FindTypeUsages::Result results = findUsages(typeName, targetValue);
+        for (const SourceLocation &loc : results) {
+            usages.append(Usage(sourceFile,
+                                matchingLine(loc.offset, doc->source()),
+                                loc.startLine,
+                                loc.startColumn - 1,
+                                loc.length));
         }
     }
     return usages;
@@ -943,9 +986,9 @@ void FindReferences::displayResults(int first, int last)
     // the first usage is always a dummy to indicate we now start searching
     if (first == 0) {
         Usage dummy = m_watcher.future().resultAt(0);
-        const QString replacement = dummy.path;
+        const QString replacement = dummy.path.toUrlishString();
         const QString symbolName = dummy.lineText;
-        const QString label = tr("QML/JS Usages:");
+        const QString label = Tr::tr("QML/JS Usages:");
 
         if (replacement.isEmpty()) {
             m_currentSearch = SearchResultWindow::instance()->startNewSearch(
@@ -959,16 +1002,16 @@ void FindReferences::displayResults(int first, int last)
                     this, &FindReferences::onReplaceButtonClicked);
         }
         connect(m_currentSearch.data(), &SearchResult::activated,
-                [](const Core::SearchResultItem& item) {
+                [](const Utils::SearchResultItem &item) {
                     Core::EditorManager::openEditorAtSearchResult(item);
                 });
-        connect(m_currentSearch.data(), &SearchResult::cancelled, this, &FindReferences::cancel);
+        connect(m_currentSearch.data(), &SearchResult::canceled, this, &FindReferences::cancel);
         connect(m_currentSearch.data(), &SearchResult::paused, this, &FindReferences::setPaused);
         SearchResultWindow::instance()->popup(IOutputPane::Flags(IOutputPane::ModeSwitch | IOutputPane::WithFocus));
 
-        FutureProgress *progress = ProgressManager::addTask(
-                    m_watcher.future(), tr("Searching for Usages"),
-                    QmlJSEditor::Constants::TASK_SEARCH);
+        FutureProgress *progress = ProgressManager::addTask(m_watcher.future(),
+                                                            Tr::tr("Searching for Usages"),
+                                                            "QmlJSEditor.TaskSearch");
         connect(progress, &FutureProgress::clicked, m_currentSearch.data(), &SearchResult::popup);
 
         ++first;
@@ -980,11 +1023,12 @@ void FindReferences::displayResults(int first, int last)
     }
     for (int index = first; index != last; ++index) {
         Usage result = m_watcher.future().resultAt(index);
-        m_currentSearch->addResult(result.path,
-                                 result.line,
-                                 result.lineText,
-                                 result.col,
-                                 result.len);
+        Utils::SearchResultItem item;
+        item.setFilePath(result.path);
+        item.setLineText(result.lineText);
+        item.setMainRange(result.line, result.col, result.len);
+        item.setUseTextEditorFont(true);
+        m_currentSearch->addResult(item);
     }
 }
 
@@ -992,7 +1036,7 @@ void FindReferences::searchFinished()
 {
     if (m_currentSearch)
         m_currentSearch->finishSearch(m_watcher.isCanceled());
-    m_currentSearch = 0;
+    m_currentSearch = nullptr;
     emit changed();
 }
 
@@ -1004,21 +1048,24 @@ void FindReferences::cancel()
 void FindReferences::setPaused(bool paused)
 {
     if (!paused || m_watcher.isRunning()) // guard against pausing when the search is finished
-        m_watcher.setPaused(paused);
+        m_watcher.setSuspended(paused);
 }
 
-void FindReferences::onReplaceButtonClicked(const QString &text, const QList<SearchResultItem> &items, bool preserveCase)
+void FindReferences::onReplaceButtonClicked(const QString &text,
+                     const Utils::SearchResultItems &items, bool preserveCase)
 {
-    const QStringList fileNames = TextEditor::BaseFileFind::replaceAll(text, items, preserveCase);
+    const Utils::FilePaths filePaths = TextEditor::BaseFileFind::replaceAll(text,
+                                                                            items,
+                                                                            preserveCase);
 
     // files that are opened in an editor are changed, but not saved
-    QStringList changedOnDisk;
-    QStringList changedUnsavedEditors;
-    foreach (const QString &fileName, fileNames) {
-        if (DocumentModel::documentForFilePath(fileName))
-            changedOnDisk += fileName;
+    Utils::FilePaths changedOnDisk;
+    Utils::FilePaths changedUnsavedEditors;
+    for (const Utils::FilePath &filePath : filePaths) {
+        if (DocumentModel::documentForFilePath(filePath))
+            changedOnDisk += filePath;
         else
-            changedUnsavedEditors += fileName;
+            changedUnsavedEditors += filePath;
     }
 
     if (!changedOnDisk.isEmpty())
